// **********************************************************************
//  This file contains PicRawPicture implementation
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <string.h>
#include <palloc.hxx>
#include <PicRawPicture.hxx>

MAKE_PICOBJECTCLASS(PicRawPicture, PicObject)

PicRawPicture::PicRawPicture() 
{
  rawImage.pixelPtr = NULLPTR;
  rawImage.pixelSize = 3;
  rawImage.offset[0] = 0;
  rawImage.offset[1] = 1;
  rawImage.offset[2] = 2;
  rawImage.height = 0;
  rawImage.width = 0;
  rawImage.pitch = 0;
}

PicRawPicture::~PicRawPicture() 
{
  setNull();
}

PicRawPicture::PicRawPicture(const PicRawPicture &other)
{
  rawImage.pixelPtr = NULLPTR;
  rawImage.height = 0;
  rawImage.width = 0;
  rawImage.pitch = 0;
  *this = (const PicObject &)other;
}

PicObject& PicRawPicture::operator = (const PicObject& obj)
{
  if(&obj == this)
    return *this;

  if(obj.IS_A(PicRawPicture)){
    const PicRawPicture &other = AS(PicRawPicture, obj);

    setNull();
    rawImage = other.rawImage;

    if(other.rawImage.pixelPtr){
      PICULONG size = rawImage.width * rawImage.height * rawImage.pixelSize;
      rawImage.pixelPtr = (unsigned char *)picAllocate(size+1);
      if(rawImage.pixelPtr){
        memcpy(rawImage.pixelPtr, other.rawImage.pixelPtr, size);
      }
    }
  }
  return *this;
}

PICUSHORT PicRawPicture::getWidth() const
{
  return rawImage.width;
}

void PicRawPicture::setWidth(PICUSHORT newWidth)
{
  rawImage.width = newWidth;
  rawImage.pitch = rawImage.width*3;
}

PICUSHORT PicRawPicture::getHeight() const
{
  return rawImage.height;
}

void PicRawPicture::setHeight(PICUSHORT newHeight)
{
  rawImage.height = newHeight;
}

unsigned char* PicRawPicture::getDataPtr() const
{
  return rawImage.pixelPtr;
}

void PicRawPicture::setDataPtr(unsigned char* newPtr)
{
  rawImage.pixelPtr = newPtr;
}

PICBOOL PicRawPicture::operator == (const PicObject& other) const
{
  return (this == &other);
}

PICBOOL PicRawPicture::operator <  (const PicObject&) const
{
  undefined("PicRawPicture::operator <");
  return boolFalse;
}

PICBOOL PicRawPicture::isNull() const
{
  return rawImage.pixelPtr == NULLPTR;
}

void PicRawPicture::setNull()
{
  if(rawImage.pixelPtr)
    picFree(rawImage.pixelPtr);
  rawImage.pixelPtr = NULLPTR;
  rawImage.height = 0;
  rawImage.width = 0;
  rawImage.pitch = 0;
}

BaseString PicRawPicture::getImage() const
{
  return asString(rawImage.width) & " * " & asString(rawImage.height);
}

void PicRawPicture::setImage(const BaseString& image)
{
  undefined("PicRawPicture::setImage");
}

// **********************************************************************

MAKE_PICOBJECTCLASS(PicRGBPicture, PicRawPicture)

PicRGBPicture::PicRGBPicture()
{

}

PicRGBPicture::PicRGBPicture(PICULONG width,
                              PICULONG height,
                              unsigned char *dataBuf)
{
  PICULONG size=width*height*3;
  setNull();

  if(size>0 && (rawImage.pixelPtr = (unsigned char *)picAllocate(size+1))){
    rawImage.width = width;
    rawImage.height = height;
    memcpy(rawImage.pixelPtr, dataBuf, size);
  }else{
    rawImage.width = 0;
    rawImage.height = 0;
  }
  rawImage.pitch=width*3;
  rawImage.pixelSize=3;
  rawImage.offset[0]=0;
  rawImage.offset[1]=1;
  rawImage.offset[2]=2;
}

PicRGBPicture::PicRGBPicture(Tk_PhotoImageBlock *block)
{
  PICULONG size=block->width*block->height*3;

  setNull();
  if(!block)
    return;

  if(size>0 && (rawImage.pixelPtr = (unsigned char *)picAllocate(size+1))){
    rawImage.width=block->width;
    rawImage.height=block->height;
    memcpy(rawImage.pixelPtr, block->pixelPtr, size);
  }else{
    rawImage.width = 0;
    rawImage.height = 0;
  }
  rawImage.pitch=block->pitch;
  rawImage.pixelSize=block->pixelSize;
  rawImage.offset[0]=block->offset[0];
  rawImage.offset[1]=block->offset[1];
  rawImage.offset[2]=block->offset[2];
}

