// **********************************************************************
//  This file contains PicSketchAlgorithm and 
//  PicSegmSignature implementations
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <PicSketchAlg.hxx>
#include <mmain.hxx>

MAKE_PICABSOBJECTCLASS(PicSegmSignature, PicSignature)

PicSegmSignature::PicSegmSignature()
  : segmAlgId(0)
{ 
}

PicSegmSignature::~PicSegmSignature()
{ 
}

PICBOOL PicSegmSignature::isNull() const
{
  return segmAlgId==0 && segmentSigs.isEmpty();
}

void PicSegmSignature::setNull()
{
  segmAlgId=0;
  segmentSigs.deleteAll();
}

BaseString PicSegmSignature::getImage() const
{
  BaseString  ret;
  PicIterator iter;
  PicSegmentAlgorithm *alg = NULL;//PicMain::globData.getSegmentAlg(segmAlgId);

  ret = "Using segmentation algorithm: id=" & asString(segmAlgId);
  if(alg)
    ret.append(" name=" & alg->name);

  ret.append("\n");

  ITERATE(segmentSigs, iter)
  {
    const PicSignature &curr = AS(PicSignature, segmentSigs.at(iter));
    ret.append("\nFor segment " & asString(segmentSigs.indexAt(iter)) & ":\n");
    ret.append(curr.getImage());
  }
  return ret;
}

char *PicSegmSignature::getBin(PICULONG &size) const
{
  size = 0;
  return NULLPTR;
}

PICBOOL PicSegmSignature::setBin(const char *data, PICULONG size)
{
  return boolFalse;
}

// **********************************************************************

MAKE_PICABSOBJECTCLASS(PicSketchAlgorithm, PicSearchAlgorithm)

PicSketchAlgorithm::PicSketchAlgorithm()
{
}

PicSignature *PicSketchAlgorithm::computeSignature(PicPicture &pic)
{
  return computeSegmSignatures(pic);
}

