// **********************************************************************
//
//      PICClient.cpp
//
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//
// General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <PICClient.h>
#include <mmain.hxx>

#include <tcl.h>

#include <message.h>
#include <iostream.h>

// =============================================================

PICNet PICClient::net;

xdrHandler *PICClient::xdrH;

bool PICClient::connected = false;
long PICClient::curDb = 0;
long PICClient::latDb = 0;
// Clear the pointer to the PICCache related information structure
cacheArrayDescr *PICClient::descArray = NULL;

// Information about server's databases and users.
PICServerData PICClient::svrData; 

#ifndef TCLTK_80
Tcl_File serverSocketForTcl;
int TclFd;
#endif

// =============================================================

void serverSocketHandler(ClientData clientData, int mask) {
  if(PicMain::threadRunning() == false) {
    while(PICClient::ReadServerMessage() == true);
  }
}

PICClient::PICClient() {
  signal(SIGPIPE,SignalHandler);
}

PICClient::~PICClient() {
}

void PICClient::SignalHandler(int a) {
  if (a == SIGPIPE)
    cout << "!!! PICClient caught SIGPIPE. !!!" << endl;   
}


// =============================================================
// 
//   CLIENT'S REQUESTS
//
// =============================================================

int PICClient::Connect(BaseString serverAddress,
			 BaseString userName,
			 BaseString password)
{
  int reply;
  DEBUG_MSG("[CLNT] Connecting..."<<endl); 
  INFO_PRINT("[CLNT] Connecting...\n"); 

  reply = HandleConnection((char *)serverAddress.chars(), PORTNUMBER, 
                           (char *)userName.chars(), 
                           (char *)password.chars());

  if (reply == AUTHORIZED) {

    PicMain::globData.picPars->addServer((char *)serverAddress.chars());
    PicMain::globData.picPars->addUser(userName);

    // Users name.
    svrData.userName = userName;

    // Read Server's InitMessage synchronously from the server.
    init_msg init;

    DEBUG_MSG("Trying to receive init..\n");

    if(!xdrH->receive(&init)) {
      cerr << "failed to receive init!" << endl;
      return UNAUTHORIZED;
    }

    ReceiveInit(init);

    DEBUG_MSG("[CLNT] Connect: Reading InitMessage...Ok!" << endl);
      
    connected = true;

#ifdef TCLTK_80
    Tcl_CreateFileHandler(net.serverSocketID(), TCL_READABLE,
			  serverSocketHandler, NULL);
#else
    TclFd=net.serverSocketID();
    
    serverSocketForTcl=Tcl_GetFile((void *)TclFd, TCL_UNIX_FD);
    Tcl_CreateFileHandler(serverSocketForTcl, TCL_READABLE, 
			  serverSocketHandler, NULL);
#endif
  }
  DEBUG_MSG("[CLNT] Leaving connect method..." << endl);
  
  return reply;
}

// =============================================================

int PICClient::HandleConnection(char* host, int port, char* userName, 
                                char* passWord)
{
  int logValue;

  DEBUG_MSG("[CLNT] In HandleConnection.....\n");
  if (!net.ClientSocketInit(host, port)) 
  {
    DEBUG_MSG("Socket initialization error (host:"<<host<<")" << endl);
    return  SOCKETINITERROR; 
  }
  xdrH=new xdrHandler(net.serverSocketID());

  //use addUser_msg to register a new user.
  struct addUser_msg usr;
  usr.name=userName;
  usr.passW=passWord;
  xdrH->send(&usr);

  DEBUG_MSG("addUser sent\n");

  struct ack_msg ack;
  xdrH->receive(&ack);

  DEBUG_MSG("ack received\n");

  switch(ack.value) {
    case ack_OK: 
      break;
    case ack_unauthorised: 
      return UNAUTHORIZED;
    case ack_busy: 
      return SERVERBUSY;
    defult: 
      break;
  }
  xdrH->free(&ack);

  DEBUG_MSG("HandleConnection OK!\n");

  return AUTHORIZED;
}

 // =============================================================

PICBOOL PICClient::Disconnect(void)
{
  DEBUG_MSG("[CLNT] Disconnecting..." << endl);
  connected = false;
#ifdef TCLTK_80
  Tcl_DeleteFileHandler(PICClient::net.serverSocketID());
#else
  Tcl_DeleteFileHandler(serverSocketForTcl);
  Tcl_FreeFile(serverSocketForTcl);
#endif

  PurgeDataStructures();
  PicMain::globData.reInitializeCache(PicMain::globData.picPars->cacheSize);
  if (Shutdown())
    return boolTrue;
  return boolFalse;
}

// =============================================================

bool PICClient::Shutdown()
{
  disconnect_msg msg;

  xdrH->send(&msg);
  DEBUG_MSG("[CLNT] Disconnect message sent.\n");  
  CloseConnection();
  return true;
}

// =============================================================

void PICClient::TimeOut(void)
{
  DEBUG_MSG("[CLNT] TimeOut." << endl);
  connected = false;
#ifdef TCLTK_80
  Tcl_DeleteFileHandler(PICClient::net.serverSocketID());
#else
  Tcl_DeleteFileHandler(serverSocketForTcl);
  Tcl_FreeFile(serverSocketForTcl);
#endif
  
  PurgeDataStructures();
  CloseConnection();
}

// =============================================================
// =============================================================

PicPicture *PICClient::ReadImageFromDb(PICULONG imageId, PICULONG indexAt) 
{
  PicPicture *pic=PicMain::getExamplePic(indexAt);
  if(!pic)
  	return(NULL);

  DEBUG_MSG("[CLNT] in ReadImageFromDb...\n");  
  
  // Read from the latest query database.
  if (!GetPicture(latDb, imageId))
    return(NULL);

  picData_msg msg;

  if(!xdrH->receive(&msg)) {
    DEBUG_MSG("[CLNT] ReadImageFromDb: failed to receive picData\n");
    return(NULL);
  } 

  DEBUG_MSG("[CLNT] GetPicture: Got image with " << msg.picData.picData_len << 
	    " bytes" << endl);
        
  DEBUG_MSG("[CLNT] ReadImageFromDb: Reading picture data..." << endl);  
  PictureData(msg, *pic);
  pic->id = msg.picID;
  pic->DBid = latDb;
  DEBUG_MSG("[CLNT] Reading picture - OK!" << endl);  
  DEBUG_MSG("[CLNT] Image ID: " << pic->id << " DBid: " << pic->DBid <<endl);
  
  return(pic);
}

// =============================================================

PICBOOL PICClient::InsertImageToDb(PicPicture &pic) {
  DEBUG_MSG("[CLNT] InsertImageToDb... curDb = " << curDb << endl);  
  DEBUG_MSG("[CLNT] Picture "<< pic.filename.chars() << endl);  
  
  // Insert into the current database.
  if (pic.origData.isNull()) {
    DEBUG_MSG("[CLNT] pic.origData NULL!\n");  
    return boolFalse;
  }

  // Strip the path part of the fileName.
  // </usr/dev/pallo.jpg> to <pallo.jpg>
  BaseString fileName = pic.filename;
  fileName = fileName.reverse();
  fileName.deleteAfter(fileName.positionOf('/') - 1);
  fileName = fileName.reverse();

  DEBUG_MSG("[CLNT] (InsertImageToDb) Size of inserted picture = ");
  DEBUG_MSG(pic.origData.size() << endl);  

  DEBUG_MSG("[CLNT] (InsertImageToDb) PicID = " << pic.id << endl);

  // Send picture.
  if(!InsertPicture(fileName.chars(), pic.origData.size(), 
		    (char *)pic.origData.get(), pic.par1.chars(), 
		    pic.par2.chars(), curDb, pic.id)) {
    DEBUG_MSG("[CLNT] InsertPicture failed!" << endl);
    return boolFalse;
  }  
  DEBUG_MSG("[CLNT] Read Id of the inserted image...\n");  

  newPic_msg reply;

  if(!xdrH->receive(&reply)) {
    DEBUG_MSG("[CLNT] InsertImageToDb failed to receive newPic" << endl);
    xdrH->proceed(); //skip thu unexpected reply.
    INFO_PRINT("FAILURE in insert.\n");
	return false;

  }

  pic.id = reply.picID;
  pic.DBid = curDb; 
  
  DEBUG_MSG("[CLNT] pic.id = " << pic.id << endl);
  DEBUG_MSG("[CLNT] PICTURE INSERTED SUCCESSFULLY!!!!!!" << endl);

  INFO_PRINT("Picture inserted ok.\n");
  
  return true;
}

// =============================================================

bool PICClient::InsertPicture(const char *filename, long size, 
                              const char *data, const char *param1, 
                              const char *param2, long dbId, long picId) {
  insertPic_msg msg;
  
  DEBUG_MSG("[CLNT] (InsertPicture) Size of inserted picture = ");
  DEBUG_MSG(size << endl);

  msg.fileName=(char *)filename;
  msg.picData.picData_len=(unsigned int)size;
  msg.picData.picData_val=(char *)data;
  msg.paramStr1=(char *)param1;
  msg.paramStr2=(char *)param2;
  msg.picDB=(databaseID_t)dbId;
  msg.picID=(pictureID_t)picId;

  DEBUG_MSG("[CLNT] dbId = " << dbId << endl);  
  DEBUG_MSG("[CLNT] (InsertPicture) Write: PicID = " << picId << endl);
  xdrH->send(&msg);
  DEBUG_MSG("[CLNT] InsertPicture message sent.\n");  
  return true;
}

// =============================================================

PICBOOL PICClient::DeleteImageFromDb(PICULONG imageId)
{
  DEBUG_MSG("[CLNT] DeleteImageFromDb...\n");  
  // Delete from the latest query database.
  if (DeletePicture(latDb, imageId))
    return boolTrue;
  return boolFalse;
}

// =============================================================

bool PICClient::GetPicture(long databaseId, long pictureId)
{
  getPic_msg msg;

  msg.picDB=(databaseID_t)databaseId;
  msg.picID=(pictureID_t)pictureId;
  xdrH->send(&msg);
  DEBUG_MSG("[CLNT] GetPicture message sent.\n");
  return true;
} 

// =============================================================

bool PICClient::DeletePicture(long databaseId, long pictureId)
{
  deletePic_msg msg;

  msg.picDB=(databaseID_t)databaseId;
  msg.picID=(pictureID_t)pictureId;
  xdrH->send(&msg);
  DEBUG_MSG("[CLNT] DeletePicture message sent.\n");
  return true;
} 

// =============================================================
// =============================================================

BaseStringArray PICClient::GetDbAlgorithms(BaseString dbName)
{
  DEBUG_MSG("[CLNT] GetDbAlgorithms for " << dbName.chars() << endl);  
  BaseStringArray algNames;
  PicIterator iter, iter2, iter3;
  long algId;
  
  if (dbName.isNull())
  {
    ITERATE(svrData.algorithms, iter)
    {
      PICDbAlgorithm &dbAlg = 
        AS(PICDbAlgorithm, svrData.algorithms.at(iter));

      algNames.add(dbAlg.name.chars());  
    }
  }
  else
  {
    ITERATE(svrData.databases, iter)
    {
      PICServerDbData &dbData = 
        AS(PICServerDbData, svrData.databases.at(iter));

      if (dbData.name == dbName)
      { 
        curDb = dbData.id;          
        DEBUG_MSG("[CLNT] GetDbAlgorithms curDb = " << dbData.id << endl);  
        ITERATE(dbData.dbAlgIds, iter2)
        {
          PICDbAlgorithm &dbAlg = 
            AS(PICDbAlgorithm, dbData.dbAlgIds.at(iter2));
		  
//		  cout << "DBAlg: " << dbAlg.id << "\n";

          ITERATE(svrData.algorithms, iter3)
          {
            PICDbAlgorithm &svrAlg = 
              AS(PICDbAlgorithm, svrData.algorithms.at(iter3));

//			cout << "ALG: " << svrAlg.id << " " << svrAlg.name.chars() << "\n";

            if (svrAlg.id == dbAlg.id)
			{
				cout << "[CLNT] Added " << svrAlg.name.chars() << "\n";
				
				algNames.add(svrAlg.name.chars());  
			}
		  }
        }
      }
    }
  }

  return algNames;
}

// =============================================================

bool PICClient::AddAlgorithm(char *libPath, char *paramWinPath,
							 int algClass)
{
	addAlg_msg msg;

    msg.algLibPath=libPath;
    msg.paramWinPath=paramWinPath;
    msg.algClass=algClass;

    xdrH->send(&msg);
    DEBUG_MSG("[CLNT]: addAlgorithm message sent.\n");

	return(true);
}

// =============================================================

bool PICClient::RemoveAlgorithm(char *algName)
{
	removeAlg_msg msg;
    int algID=0;
	
	PICDbAlgorithm *tmp=PicMain::getSearchAlg(algName);
	if(tmp)
		algID=tmp->id;

	if(!algID)
		return(false);

    msg.ID = algID;
    
	xdrH->send(&msg);
    DEBUG_MSG("[CLNT]: removeAlgorithm message sent.\n");

	return(true);
}

// =============================================================

bool PICClient::ModifyAlgorithm(char *libPath, char *paramWinPath,
				char *algName, int algClass, bool recalcSigs)
{
	modifyAlg_msg msg;
	int algID=0;

    PicIterator iter;
    PICDbAlgorithm *tmp=PicMain::getSearchAlg(algName);
	if(tmp)
		algID=tmp->id;

	if(!algID)
		return(false);

	msg.algLibPath=libPath;
	msg.paramWinPath=paramWinPath;
	msg.ID=algID;
	msg.algClass=algClass;
	msg.recalcSigs=recalcSigs;
	
	xdrH->send(&msg);
	DEBUG_MSG("[CLNT]: modifyAlgorithm message sent.\n");

	return(true);
}
	

// =============================================================

bool PICClient::setDatabaseAlgorithms(char *dbName, PicArray &algArray)
{
  modifyAlgs_msg msg;
  PicIterator iter;
  int databaseId=0, i=0;

  DEBUG_MSG("[CLNT] setDatabaseAlgorithms...\n");  
	
  msg.targetDB=dbName;
  msg.usedAlgs.usedAlgs_len=(unsigned int)algArray.size();
  msg.usedAlgs.usedAlgs_val=new algorithmID_t[msg.usedAlgs.usedAlgs_len];

  DEBUG_MSG("[CLNT] Algs in use " << algArray.size() << endl);

  ITERATE(algArray, iter)
  {
	PicString &algName = 
	  AS(PicString, algArray.at(iter));

	PICDbAlgorithm *tmp=PicMain::getSearchAlg(algName);
	if(tmp)
	{
	    msg.usedAlgs.usedAlgs_val[i++]=(algorithmID_t)tmp->id;
  
	    DEBUG_MSG("[CLNT] Set alg " << tmp->id);
		DEBUG_MSG(" : " << algName.chars() << endl);
    }
  }
  
  xdrH->send(&msg);

  DEBUG_MSG("[CLNT] modifyAlgs message sent.\n");  

  delete[] msg.usedAlgs.usedAlgs_val;
  
  return true;
}

// =============================================================
// =============================================================

BaseStringArray PICClient::GetDbNames(void) {
  DEBUG_MSG("[CLNT] GetDbNames...\n");  
  BaseStringArray dbNames;
  PicIterator iter;

  ITERATE(svrData.databases, iter) {
    PICServerDbData &dbData = 
      AS(PICServerDbData, svrData.databases.at(iter));
                           
    dbNames.add(dbData.name.chars());
    DEBUG_MSG("[CLNT] GetDbNames : databaseName = ");
    DEBUG_MSG(dbData.name.chars() << endl); 
  }
  return dbNames;
}

// =============================================================

BaseStringArray PICClient::GetAllDbNames(void) {
  DEBUG_MSG("[CLNT] GetAllDbNames...\n");  
  BaseStringArray dbNames;
  PicIterator iter;

  ITERATE(svrData.allDatabases, iter) {
    PICServerDbData &dbData = 
      AS(PICServerDbData, svrData.allDatabases.at(iter));
                           
    dbNames.add(dbData.name.chars());
    DEBUG_MSG("[CLNT] GetAllDbNames : databaseName = ");
    DEBUG_MSG(dbData.name.chars() << endl); 
  }
  return dbNames;
}

// =============================================================

bool PICClient::CreateDatabase(char *path, char *databaseName) {
  //Making sure that there is not space characters in databaseName

  for(int i=0;databaseName[i];++i) {
    if(databaseName[i]==' ') 
      databaseName[i]='_';
  }

  DEBUG_MSG("[CLNT] CreateDatabase....\n");

  createDB_msg msg;

  msg.directory=path;
  msg.DBName=databaseName;
  DEBUG_MSG("[CLNT] CreateDatabase: creating DB " << databaseName <<
	    " in directory " << path << endl);

  msg.algs.algs_len=svrData.algorithms.size();
  msg.algs.algs_val=new algorithmID_t[msg.algs.algs_len];

  // Write algorithm ids.
  PicIterator iter;
  int i=0;
  ITERATE(svrData.algorithms, iter) {
    PICDbAlgorithm &alg = AS(PICDbAlgorithm, svrData.algorithms.at(iter));
    DEBUG_MSG("[CLNT] CreateDatabase: adding algorithm with ID " <<
	      alg.id << endl);
    msg.algs.algs_val[i++]=alg.id;
  }
  xdrH->send(&msg);
  DEBUG_MSG("[CLNT] CreateDatabase message sent.\n");

  delete msg.algs.algs_val;
  return true;
}        

// =============================================================

bool PICClient::DeleteDatabase(BaseString dbName)
{
  DEBUG_MSG("[CLNT] DeleteDatabase...\n");  
  deleteDB_msg msg;

  msg.DBName=(char *)dbName.chars();
  xdrH->send(&msg);
  DEBUG_MSG("[CLNT] DeleteDatabase message sent.\n");

  PicIterator dbIter;
  ITERATE(svrData.databases, dbIter)
  {
    PICServerDbData &dbData =
      AS(PICServerDbData, svrData.databases.at(dbIter));

    if(dbData.name==dbName)
    {
        svrData.databases.deleteAt(dbIter);
        break;
    }
  }

  return true;
}

// =============================================================

bool PICClient::GetServerOptions()
{
  getServOpt_msg msg;
  
  DEBUG_MSG("[CLNT] GetServerOptions...\n");  
  xdrH->send(&msg);
  return true;
}  

// =============================================================

bool PICClient::SetServerOptions(char *maxClients, char *timeout, 
                                 char *httpAddress, char *commercial) {
  DEBUG_MSG("[CLNT] SetServerOptions...\n");  
  setServOpt_msg msg;
  
  //I wanna set 4 options.
  msg.opts.opts_len=4;
  msg.opts.opts_val=new namedValue[msg.opts.opts_len];

  msg.opts.opts_val[0].name="maxclients";
  msg.opts.opts_val[0].value=maxClients;

  msg.opts.opts_val[1].name="timeout";
  msg.opts.opts_val[1].value=timeout;

  msg.opts.opts_val[2].name="http";
  msg.opts.opts_val[2].value=httpAddress;

  msg.opts.opts_val[3].name="info";
  msg.opts.opts_val[3].value=commercial;

  xdrH->send(&msg);

  delete msg.opts.opts_val;

  DEBUG_MSG("[CLNT] SetServerOptions message sent.\n");  

  return true;
}

// =============================================================

BaseString PICClient::GetAddress() 
{
  DEBUG_MSG("[CLNT] in get address: ");
  DEBUG_MSG(svrData.httpAddress.chars() << endl);
  return svrData.httpAddress;
}

// =============================================================

BaseString PICClient::GetServerAdvertisement() 
{
  DEBUG_MSG("[CLNT] in get server advertisement: ");
  DEBUG_MSG(svrData.commercial.chars() << endl);
  return svrData.commercial;
}

// =============================================================

BaseString PICClient::GetAlgorithmInfo(char* alg)
{
  DEBUG_MSG("[CLNT] in get algorithm info" << endl;);

  BaseString algName = alg, result;

  PICDbAlgorithm *tmp=PicMain::getSearchAlg(algName);
  if(tmp)
  	return(tmp->description);

  return result;
}

// =============================================================

bool PICClient::PurgeDataStructures()
{
  DEBUG_MSG("[CLNT] PurgeDataStructures..." << endl);  

  PicIterator dbIter, dbIter2, algIter, usrIter, usrIter2;

  // Purge variables.
  svrData.ipAddress.setEmpty();
  svrData.portNbr = 0;
  svrData.name.setEmpty();
  svrData.httpAddress.setEmpty();
  svrData.commercial.setEmpty();
  svrData.userName.setEmpty();
  svrData.authLevel = 0;
  svrData.canChangePassword = 0;

  // Purge databases array.
  ITERATE(svrData.databases, dbIter)
  {
    PICServerDbData &dbData = 
      AS(PICServerDbData, svrData.databases.at(dbIter));

    dbData.name.setEmpty();
    dbData.id = 0;

    ITERATE(dbData.dbAlgIds, dbIter2)
    {
      PICDbAlgorithm &dbAlg = 
        AS(PICDbAlgorithm, dbData.dbAlgIds.at(dbIter2));
      dbAlg.name.setEmpty();
      dbAlg.id = 0;
      dbAlg.description.setEmpty();
    }
    dbData.dbAlgIds.deleteAll();

    dbData.userRightsLevel = 0;
  }
  svrData.databases.deleteAll();

  // Purge algorithms array.
  ITERATE(svrData.algorithms, algIter)
  {
    PICDbAlgorithm &dbAlg = 
      AS(PICDbAlgorithm, svrData.algorithms.at(algIter));

    dbAlg.name.setEmpty();
    dbAlg.id = 0;
    dbAlg.description.setEmpty();
  }
  svrData.algorithms.deleteAll();

  // Purge users array.
  ITERATE(svrData.users, usrIter)
  {
    PICUserInfo &usrInfo = 
      AS(PICUserInfo, svrData.users.at(usrIter));

    ITERATE(usrInfo.dbRights, usrIter2)
    {
      PICDbRight &dbRight = 
        AS(PICDbRight, usrInfo.dbRights.at(usrIter2));

      dbRight.id = 0;
      dbRight.authLevel = 0;
    }
    usrInfo.dbRights.deleteAll();
  }
  svrData.users.deleteAll();

  // Purge allDatabases array.
  ITERATE(svrData.allDatabases, dbIter)
  {
    PICServerDbData &dbData = 
      AS(PICServerDbData, svrData.allDatabases.at(dbIter));

    dbData.name.setEmpty();
    dbData.id = 0;

    ITERATE(dbData.dbAlgIds, dbIter2)
    {
      PICDbAlgorithm &dbAlg = 
        AS(PICDbAlgorithm, dbData.dbAlgIds.at(dbIter2));
      dbAlg.name.setEmpty();
      dbAlg.id = 0;
      dbAlg.description.setEmpty();
    }
    dbData.dbAlgIds.deleteAll();

    dbData.userRightsLevel = 0;
  }
  svrData.allDatabases.deleteAll();
  return true;
}

// =============================================================

BaseStringArray PICClient::GetInitData()
{
  BaseStringArray iData;
  char c[2] = {0,0};
  *c = (char)('0'+svrData.authLevel);

  iData.add(c);
  iData.add((svrData.canChangePassword)? "1":"0");
  PicIterator iter;

  ITERATE(svrData.algorithms, iter)
  {
    PICDbAlgorithm &dbAlg =
      AS(PICDbAlgorithm, svrData.algorithms.at(iter));
    iData.add(dbAlg.name.chars());
    //added tk (the parameters of algorithm) 
    iData.add(dbAlg.tk.chars());
  }
  return iData;
}
                                       
// =============================================================
// 
//   RECEIVE METHOD
//
// =============================================================

void handleAck(ackValue value) {
  LOG_MSG("handleAck with " << (int)ackValue << endl);
}

int PICClient::xdrTranslate(void) {
  cout << "in xdrTranslate. foo!" << endl;

  switch(xdrH->currentMessage()) {

  case msg_ack:
    ack_msg ackMsg;
    LOG_MSG("[XDR] received ack\n");
    xdrH->receive(&ackMsg);
    handleAck(ackMsg.value);
    xdrH->free(&ackMsg);
    break;

  case msg_algProgress:
    algProgress_msg algProgressMsg;
    LOG_MSG("[XDR] received algProgress\n");
    xdrH->receive(&algProgressMsg);
    AlgorithmProgressReport(algProgressMsg);
    xdrH->free(&algProgressMsg);
    break;

  case msg_newDB:
    newDB_msg newDBMsg;
    LOG_MSG("[XDR] received newDB\n");
    xdrH->receive(&newDBMsg);
    NewDatabase(newDBMsg);
    xdrH->free(&newDBMsg);
    break;

  case msg_newUser:
    newUser_msg newUserMsg;
    LOG_MSG("[XDR] received newUser\n");
    xdrH->receive(&newUserMsg);
    NewUserRights(newUserMsg);
    xdrH->free(&newUserMsg);
    break;

  case msg_progReport:
    progReport_msg progReportMsg;
    LOG_MSG("[XDR] received progReport\n");
    xdrH->receive(&progReportMsg);
    QueryProgressReport(progReportMsg);
    xdrH->free(&progReportMsg);
    break;

  case msg_queryResults:
    queryResults_msg queryResultsMsg;
    LOG_MSG("[XDR] received queryResults\n");
    xdrH->receive(&queryResultsMsg);
    QueryResults(queryResultsMsg);
    xdrH->free(&queryResultsMsg);
    break;

  case msg_servOpts:
    servOpts_msg servOptsMsg;
    LOG_MSG("[XDR] received servOpts\n");
    xdrH->receive(&servOptsMsg);
    ServerOptions(servOptsMsg);
    xdrH->free(&servOptsMsg);
    break;

  case msg_userData:
    userData_msg userDataMsg;
    LOG_MSG("[XDR] received userData\n");
    xdrH->receive(&userDataMsg);
    UserData(userDataMsg);
    xdrH->free(&userDataMsg);
    break;

  case msg_init:
    init_msg initMsg;
    LOG_MSG("[XDR] received init\n");
    xdrH->receive(&initMsg);
    ReceiveInit(initMsg);
    xdrH->free(&initMsg);
    break;

    /*case msg_: //TEMPLATE
    _msg Msg;
    LOG_MSG("[XDR] received \n");
    xdrH->receive(&Msg);
    (Msg);
    xdrH->free(&Msg);
    break;*/

  case msg_queryNoResults:
    queryNoResults_msg queryNoResultsMsg;
	LOG_MSG("[XDR] received queryNoResults\n");
  	xdrH->receive(&queryNoResultsMsg);
	PicGUI::NoQueryMatches();	
	xdrH->free(&queryNoResultsMsg);
	break;

  default:
    DEBUG_MSG("xdrTranslate: unknown message!\n");
    xdrH->proceed();
    return 1;
  }
  return 1;
}

bool PICClient::ReadServerMessage()
{
  if(!connected)
    return false;

  if(!net.CheckSocket(0))
    return false;
  DEBUG_MSG("[CLNT] in readServerMessage.\n");
  xdrTranslate();
  return true;
}

// =============================================================
// 
//   SERVER'S MESSAGES
//
// =============================================================

void PICClient::ReceiveInit(init_msg &init)
{
  unsigned int size = 0;  // Number of databases

  DEBUG_MSG("[CLNT] ReceiveInit:Reading authLevel..."<<endl);  

  svrData.authLevel=init.authLevel;    //User's authorization level to server.
  svrData.canChangePassword=init.changePW;  // Can user change password or not?

  INFO_PRINT("[CLNT] ReceiveInit: Initializing clnts data structures.\n");  
  DEBUG_MSG("[CLNT] ReceiveInit: Reading databases..." << endl);  

  // Number of databases in this server...
  size=init.DBs.DBs_len;
  DEBUG_MSG("[CLNT] ReceiveInit: Number of databases: " << size << endl);    
  if(size>0)
  {
    for(unsigned int iter=0;iter<size;iter++)
    {
      DBInfo &DBi=init.DBs.DBs_val[iter];
      PICServerDbData dbData;

      dbData.id = DBi.ID; // Database's id
      DEBUG_MSG("[CLNT]ReceiveInit:DbId = " << dbData.id << endl);  
      
      // User's authorization level to the database.
      DatabaseAuth dbaseRightLevel=DBi.authLevel; // Users rightlevel.
      dbData.userRightsLevel=dbaseRightLevel;
      DEBUG_MSG("[CLNT]ReceiveInit:userRightsLevel = ");
      DEBUG_MSG(dbData.userRightsLevel << endl);

      dbData.name = strdup(DBi.name); // Database's name
      DEBUG_MSG("[CLNT]ReceiveInit:DbName=" << dbData.name.chars() << endl);  

      // Number of algorithms supported in this database.
      unsigned int count = DBi.algs.algs_len;
      DEBUG_MSG("[CLNT] ReceiveInit: Read NumberOfAlgs = " << count << endl);  
      for (unsigned int j=0;j<count;j++) {
	PICDbAlgorithm dbAlg;
	dbAlg.id = DBi.algs.algs_val[j];
	DEBUG_MSG("[CLNT] ReceiveInit: Read alg with ID " << dbAlg.id << endl);
	dbData.dbAlgIds.add(dbAlg);
      }

      DEBUG_MSG("[CLNT] Has user an access?" << endl);  
      // Has user an access to this database.
      if(dbaseRightLevel!=DB_NoAccess) 
      {
        DEBUG_MSG("YES - user has access!" << endl);  
        // Add database to the databases array.
        svrData.databases.add(dbData);
      }
      DEBUG_MSG("[CLNT] Adding dbData to allDatabases." << endl);  
      // Add database to the allDatabases array.
      svrData.allDatabases.add(dbData);
      DEBUG_MSG("[CLNT] Adding dbData to allDatabases...DONE! " << endl);  
    }
    DEBUG_MSG("Database read Ok!" << endl);  
  }
  DEBUG_MSG("[CLNT] ReceiveInit:Reading algorithms..." << endl);  

  // Number of algorithms supported in this server.
  unsigned int count = init.algs.algs_len;
  DEBUG_MSG("[CLNT] ReceiveInit:NumberOfAlgorithms = " << count << endl);  
  if (count>0)
  {
    PicSearchType *allStype = PicMain::globData.getSearchType("All");
    for (unsigned int iter=0;iter<count;iter++)
    {
      PICDbAlgorithm dbAlg;
      AlgInfo &alg=init.algs.algs_val[iter];

      dbAlg.id = alg.ID;
      DEBUG_MSG("[CLNT]algId= " << dbAlg.id << endl);  

      dbAlg.name = strdup(alg.name);
      DEBUG_MSG("[CLNT]algName= " << dbAlg.name.chars() << endl);  

      dbAlg.description = strdup(alg.description);
      DEBUG_MSG("[CLNT]algDescr= " << dbAlg.description.chars() << endl);  
        
      //listn algoritmien tk:t*************************
      dbAlg.tk = strdup(alg.algParamWin);
      DEBUG_MSG("[CLNT]algTk= " << dbAlg.tk.chars() << endl);
      //*************************************************
      svrData.algorithms.add(dbAlg);

      // SP: no algNameTmp declared.. hope the fix below does OK.
      //  if(!(allStype->algs.contains(algNameTmp)))

      if(!(allStype->algs.contains(alg.name)))
	allStype->addAlgorithm(&dbAlg);
					
	
    }
    DEBUG_MSG("[CLNT] Algorithm read Ok!" << endl);  
  }
  
  // Server's commercial text.
  svrData.commercial = strdup(init.serverAd);
  DEBUG_MSG("[CLNT]Server's COMMERCIAL TEXT = ");
  DEBUG_MSG(svrData.commercial.chars() << endl);

  // URL of the server's homapage.
  svrData.httpAddress = strdup(init.serverHTTP);
  DEBUG_MSG("[CLNT]Server's HOMEPAGE = ");
  DEBUG_MSG(svrData.httpAddress.chars() << endl);

  // Send all the necessary information to Tcl/Tk.
  // Get information about users if the user is SuperUser.
  if (svrData.authLevel == SuperUser) {
    GetUsers();
    //FIXME: there's an out-of-sync situation if this isn't read here,
    //but this is not the best way to handle all this shit..
    xdrTranslate(); //handle whatever reply you get.
    //UserData(userDataPacket);
  }
  //GetDbNames(); // Get Databasenames of the server.
  //GetDbUsers(); // Get Usernames.

  //xdrH->free(&init); 
  //FIXME: I want to be able to free this.
}

// =============================================================

void PICClient::CloseConnection() {
  DEBUG_MSG("[CLNT] CloseConnection...\n");  
  net.CloseServerSocket();
}

// =============================================================
// 
//   Servers Replies
//
// =============================================================


void PICClient::AlgorithmProgressReport(algProgress_msg &msg)
{
  //  PicGUI::AlgorithmProgressReport(msg.opType, msg.algID);
  DEBUG_MSG("[CLNT] In AlgorithmProgressReport Method..."<<endl);   
  INFO_PRINT("[CLNT] In AlgorithmProgressReport Method...\n");   
}

// =============================================================

void PICClient::NewDatabase(newDB_msg &msg)
{
  DEBUG_MSG("[CLNT] NewDatabase...\n");  
  long numAlgs = 0;
  long databaseId = 0;

  PICServerDbData dbData; 
  // Database id
  databaseId = msg.DB.ID;
  dbData.id = databaseId; 
  // Database name
  dbData.name = msg.DB.name;

  // Number of algorithms in this server.
  numAlgs = msg.DB.algs.algs_len;

  if (numAlgs > 0) {
    for (long iter = 0; iter < numAlgs; iter++) {
      PICDbAlgorithm newDbAlg;
      newDbAlg.id = msg.DB.algs.algs_val[iter];
      dbData.dbAlgIds.add(newDbAlg);
    }
  }
  //SP FIXME: is this part correct? it skips a value in the msg.
  // Users rightLevel is write (=3).
  dbData.userRightsLevel = 3;

  // Add database to the users datastructures.
  svrData.databases.add(dbData);
  svrData.allDatabases.add(dbData); 

  if (svrData.authLevel == SuperUser) {
    PicIterator iter ;

    ITERATE(svrData.users, iter) {
      PICUserInfo &usr = AS(PICUserInfo, svrData.users.at(iter));
    
      if ((BaseString)usr.name == svrData.userName) {
        PICDbRight dbRight;
        dbRight.id = databaseId;
        dbRight.authLevel = 3;
        usr.dbRights.add(dbRight);
      }
    }
  }
  xdrH->free(&msg);
}

// =============================================================

void PICClient::ServerOptions(servOpts_msg &msg) {
  DEBUG_MSG("[CLNT] ServerOptions..."<<endl);
  
  BaseString maxClients;   // Maximum number of clients.
  BaseString timeout;      // Timeout limit.
  BaseString httpAddress;  // URL of the server's homepage.
  BaseString info;         // Commericial text of the server.
  BaseStringArray allOpts; // All the options in one bunch.

  // Server's parameter can be in arbitrary order.
  for(unsigned int iter=0;iter<msg.opts.opts_len;++iter)
  {
    namedValue &opt=msg.opts.opts_val[iter];

    //FIXME: check if basestring does a copy (== is strdup unnecessary).
    if(!strcmp(opt.name,"maxclients"))
      maxClients = strdup(opt.value); 
     
    else if(!strcmp(opt.name,"timeout"))
      timeout = strdup(opt.value);
      
    else if(!strcmp(opt.name,"http"))
      httpAddress = strdup(opt.value);
      
    else if(!strcmp(opt.name,"info"))
      info = strdup(opt.value);
      
    else
      DEBUG_MSG("[CLNT] Unknown server's parameter." << endl);
  }

  // Catenate the arguments.
  allOpts.add(maxClients);
  allOpts.add(timeout);
  allOpts.add(httpAddress);
  allOpts.add(info);

  PicGUI::setServerOptions(allOpts);
}



/*** NEW METHODS INTRODUCED IN V3.1 ***/

/*
 * This unused construction is to make identical algorithm
 * names to differ in Tk. It could have been used when the
 * algorithm data is received from the server in the init.
 *
 */
int PICClient::algNameConvToTk(int algId, char *oldName, char *newName)
{
	if(!oldName || !newName || !algId)
		return(0);

	sprintf(newName, "%d_%s", algId, oldName);

	return(1);
}

