// **********************************************************************
//
//      PICQuery.cpp
//
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//
// General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <PICClient.h>
#include <mmain.hxx>
#include <message.h>

PICBOOL PICClient::StartQuery(BaseString algName, PicPicture &pic)
{
  DEBUG_MSG("[CLNT] QUERY..." << endl);
  struct query_msg msg;
  PICDbAlgorithm *algtmp;  
  PicArray algParams, genOpts;
  long payLoad;

  if(!PicMain::getParams(genOpts))
  {
  	DEBUG_MSG("[CLNT] Unable to get general query parameters.\n");
	return boolFalse;
  }

  algtmp=PicMain::getSearchAlg(algName);
  if(!algtmp)
  {
    DEBUG_MSG("[CLNT] Query failed to find algorithm.\n");
    return boolFalse;
  }

  algParams=algtmp->params;
  msg.algID=algtmp->id;
  msg.DBID=curDb;

  cout<<"parametrien koko: "<<(algParams.size())<<"\n";

  msg.params.params_len=algParams.size();
  msg.params.params_val=new strStruct[msg.params.params_len];

  // Algorithm's parameters.
  PicIterator iter;
  unsigned int paramc=0;
  ITERATE(algParams, iter) {
    msg.params.params_val[paramc++].str =
      (char *)(AS(PicString, algParams.at(iter))).chars();    
  }

  if(!AddGeneralOptions(msg, genOpts)) {
    DEBUG_MSG("[CLNT] QUERY: adding GeneralOptions failed." << endl);
    return boolFalse;
  }

  if(pic.id)									// Do ID based query
  {
    DEBUG_MSG("[CLNT] QUERY by ID." << endl);
	msg.type=queryByID;
	msg.qPicID=pic.id;
	msg.qPicDB=pic.DBid;
   
    cout << "ID: " << pic.id << " DB ID: " << pic.DBid << endl;
	
	xdrH->send(&msg);
	
	latDb=curDb;
  
    return(true);
  }
  
  // Well then, let's do a picture based query.
  
  DEBUG_MSG("[CLNT] QUERY by picture." << endl);
  msg.type=queryByPicture;
  msg.qPicDB=msg.qPicID=0; //these not used.

  xdrH->send(&msg);

  picData_msg qPic;

  //not initialized below
  qPic.fileName=qPic.paramStr1=qPic.paramStr2="";
  qPic.picID=0;

  if(!pic.origData.isNull())
  {
  	qPic.picData.picData_len=pic.origData.size();
    qPic.picData.picData_val=new char[qPic.picData.picData_len];
    cout << "Kuvan koko " << pic.origData.size() << endl;
  
  // Picture data
    memcpy(qPic.picData.picData_val,(char *)pic.origData.get(),
      qPic.picData.picData_len);
  }
  else									// Hmm, its raw as shit
  {
  	unsigned long picSize=pic.raw->getWidth()*pic.raw->getHeight()*3;

    qPic.picData.picData_len=picSize;
	qPic.picData.picData_val=new char[picSize];

	memcpy(qPic.picData.picData_val, (char *)pic.raw->getDataPtr(), picSize);
  }	

  // Picture width in pixels
  qPic.picWidth=pic.raw->getWidth();
  cout<<"Picture width "<<pic.raw->getWidth()<<"\n";
  qPic.picHeight=pic.raw->getHeight();
  cout<<"Picture height "<<pic.raw->getHeight()<<"\n";

  // Picture format ID
  if(pic.format)
    qPic.picFormat=pic.format->id;
  else
    qPic.picFormat=0;

  //FIXME: enable cache.
  /*// If the PICCache is disabled, do not attempt to use it
  if(PicMain::globData.cacheIsDisabled == false) {
    // Free the PICCache related information structure before redoing it
    if(descArray != NULL) {
      free(descArray->descArray);
      free(descArray);
    }
    // Fill the packet with PICCache information
    descArray = PicMain::globData.cache->getArrayDescr();
    // First write how many ids the server should expect
    packet.WriteInt32(descArray->nonNull);
    // Then loop through array taking only non-null values
    for(int i = 0; i < descArray->elements; i += 2) {
      if(descArray->descArray[i] || descArray->descArray[i + 1]) {
	packet.WriteInt32(descArray->descArray[i]);
	packet.WriteInt32(descArray->descArray[i + 1]);
      }
    }
  }*/


  xdrH->send(&qPic);

  latDb=curDb;

  //FIXME: free all that memory.

  return true;
}

// =============================================================

PICBOOL PICClient::AddGeneralOptions(query_msg &msg, 
                                     PicArray &generalOptions) {
  DEBUG_MSG("[CLNT] QUERY: adding values from genOpts." << endl);

  long total_results = 0;
  long oneshot_results = 0;
  long min_match = 0;

  // Maximum result thumbnails.
  if(generalOptions.contains(PICPARAMS_NUM_OF_RESULT_IMAGES)) {
    PicString &ref = 
      AS(PicString, generalOptions.at(PICPARAMS_NUM_OF_RESULT_IMAGES));
    PicInteger newVal(ref);
    total_results = newVal.asInt();
  }

  if(!total_results)
    total_results=100;

  msg.thumbMax=total_results;
  DEBUG_MSG("[CLNT] AddGeneralOptions: thumbMax " << total_results << endl);

  if(generalOptions.contains(PICPARAMS_DB_IMAGE_FRAMES_ON_SCREEN)) {
    PicString &ref = 
      AS(PicString, generalOptions.at(PICPARAMS_DB_IMAGE_FRAMES_ON_SCREEN));
    PicInteger newVal(ref);
    oneshot_results = newVal.asInt();
  }

  if(!oneshot_results)
    oneshot_results=10;

  msg.thumbNum=oneshot_results;
  DEBUG_MSG("[CLNT] AddGeneralOptions: thumbNum " << oneshot_results << endl);

  // Minimum match percentage for an image
  if(generalOptions.contains(PICPARAMS_HIT_PERC)) {
    PicString &ref = AS(PicString, generalOptions.at(PICPARAMS_HIT_PERC));
    PicInteger newVal(ref);
    min_match = newVal.asInt();
  }

  msg.minMatch = min_match;
  DEBUG_MSG("[CLNT] AddGeneralOptions: minMatch " << min_match << endl);

  return boolTrue;
}

// =============================================================

PICBOOL PICClient::StopQuery(void) {
  stopQuery_msg msg;

  xdrH->send(&msg);
  DEBUG_MSG("[CLNT] StopQuery message sent.\n");  
  return boolTrue;
}

// =============================================================

PICBOOL PICClient::GetMoreThumbNails(int numOfFirst, PicArray &genOpts) {
  PicInteger numOfThumbNails(AS(PicString,
				genOpts.at(PICPARAMS_DB_IMAGE_FRAMES_ON_SCREEN)));
  if (GetMoreThumbnails(numOfFirst, numOfThumbNails.asInt()))
    return boolTrue;
  return boolFalse;
}

// =============================================================

void PICClient::QueryProgressReport(struct progReport_msg &msg) {
  DEBUG_MSG("[CLNT] In QueryProgressReport Method..."<<endl);   
  // Read progressvalue.
  long progressValue = msg.status;

  DEBUG_MSG("[CLNT] PROGRESSREPORT VALUE = " << progressValue << endl);   
  if (!PicGUI::QueryProgressReport(progressValue))
    DEBUG_MSG("[CLNT] PicGUI failed..." << endl);   

  DEBUG_MSG("[CLNT] QueryProgressReport sent to PicGUI..."<<endl);   
  INFO_PRINT("[CLNT] In QueryProgressReport Method...\n");   
}

// =============================================================

void PICClient::QueryResults(queryResults_msg &msg) {
  DEBUG_MSG("[CLNT] In QueryResults Method..."<<endl);

  PicArray pics;
  PicIterator iter;
  PICUSHORT numOfPics;
  PicImageFormat *format;
  unsigned int receivedPics;
  unsigned int i;
  int matchRanking = 0;
  int matchRate = 0;
  long lookupPercent;
  char *picData;
  char stringBuffer[128];

  numOfPics = PicMain::globData.picPars->dbImageFramesOnScreen;
  format = PicMain::globData.getImageFormat(PIC_JPEG_IMAGEFORMAT_ID);

  receivedPics = msg.nails.nails_len;
  DEBUG_MSG("[CLNT] number of received pics = " << receivedPics <<endl);

  // Method has gone through surgery because of the PICCache -SK
  for (i = 1; (i <= numOfPics) && (i <= receivedPics); i++) {
    if(!pics.putAt(i, PicPicture()))
      continue; //FIXME: use a proper fail
    
    const thumbNail &nail=msg.nails.nails_val[i-1]; //why does i start at 1?
    PicPicture &curr = AS(PicPicture, pics.at(i));

    curr.format = format;

    // Read the size of the thumbnail
    unsigned int blockSize = nail.data.data_len;

    if(blockSize != 0) {
      // Server sent us a thumbnail, so it's not in cache
      if(DEBUG_CACHE_ON && PicMain::globData.cacheIsDisabled == false)
	DEBUG_CACHE("Thumbnail size != 0, receiving from server.\n");
      if(PicMain::globData.cacheIsDisabled == false)
	PicMain::globData.cache->cacheMisses++;

      picData = new char[blockSize];
      memcpy(picData,nail.data.data_val,blockSize);

      PicInCharStream stream(picData, blockSize);
      PicRawPicture *newRaw;
      if((newRaw = curr.format->readPicture(stream, blockSize))) {
	DEBUG_MSG("[CLNT] Converted data to JPEG ok." << endl);
	curr.setRaw(newRaw);
      } else {
	DEBUG_MSG("[CLNT] Conversion to JPEG failed, blocksize= " << 
		  blockSize<< endl);
	//FIXME: get me an error return here, OK?
      }
      // Release allocated block. FIXME: the block is really unnecessary.
      delete picData;
    
      // Read thumbnails id
      curr.id = nail.picID;
      DEBUG_MSG("[CLNT] Thumbnail id = " << curr.id << endl);
      
      // Read pictures param 1 
      curr.par1 = strdup(nail.paramStr1);
      // Read pictures param 2
      curr.par2 = strdup(nail.paramStr2);

      // Read pictures rank
      curr.matchRanking = nail.rank;
      DEBUG_MSG("[CLNT] Thumbnail ranking = " << curr.matchRanking << endl);
      // Read pictures rate
      
      matchRate = nail.match;      
      curr.matchRate=asString(matchRate/10) & "." & 
	asString(matchRate % 10) & "%";
      DEBUG_MSG("[CLNT] Thumbnail rank = " << curr.matchRate.chars() << endl);
    } 
    else { //blockSize==0
      // If the PICCache is disabled, do not attempt to use it
      if(PicMain::globData.cacheIsDisabled == false) {
	// Since the thumbnail sent by server had size 0, lookup from cache
	PicPicture *thisRec=PicMain::globData.cache->getThumb(curDb, curr.id);
	if(thisRec != NULL) {
	  if(DEBUG_CACHE_ON) {
	    sprintf(stringBuffer, "Taking thumbnail %ld from PICCache.\n", 
		    curr.id);
	    DEBUG_CACHE(BaseString(stringBuffer));
	  }
	  thisRec->par1 = curr.par1;
	  thisRec->par2 = curr.par2;
	  thisRec->matchRanking = curr.matchRanking;
	  thisRec->matchRate = curr.matchRate;
	  pics.putAt(i, PicPicture(*thisRec));
	}
	else {
	  // Should never happen, PANIC!!
	  DEBUG_MSG("[CLNT] QueryResults: catastrophic failure!\n");
	  //FIXME: at least an error return would be in order, huh?
	}
      }
      // Display whole array, in fact just the pictures that have been received
      PicMain::getGUI().setQueryResults(pics);
    }
  }
  // If the PICCache is disabled, do not attempt to use it
  if(PicMain::globData.cacheIsDisabled == false) {
    // Now all result images have been received & displayed, update cache
    for(unsigned int picOffset = 1; picOffset <= pics.size(); picOffset++) {
      PicPicture &curr = AS(PicPicture, pics.at(picOffset));
      if(PicMain::globData.cache->isThumbInCache(curDb, curr.id) == false)
	// Not found in cache, insert
	PicMain::globData.cache->putThumb(curDb, curr.id, curr);
    }
    // Show PICCache performance
    lookupPercent = PicMain::globData.cache->cacheHits * (100 * 100);
    lookupPercent /= ((PicMain::globData.cache->cacheHits + 
		       PicMain::globData.cache->cacheMisses) * 100);
    sprintf(stringBuffer, "[CACHE] Succesful cache fetches %ld%% "
	    "(%ld hits, %ld misses)\n",lookupPercent,
	    PicMain::globData.cache->cacheHits,
	    PicMain::globData.cache->cacheMisses);
    INFO_PRINT(BaseString(stringBuffer));
    // Print PICCache allocation status, only if DEBUG_CACHE_ON
    if(DEBUG_CACHE_ON) {
      long cacheAllocation = 0;
      long cacheAllocationPercent = 0;
      for(unsigned int j = 0; j < PicMain::globData.cache->arraySize; j++) {
	long inThisChain = PicMain::globData.cache->elementsInChain(j);
	sprintf(stringBuffer, "Size of chain %d is %ld\n", j, inThisChain);
	DEBUG_CACHE(BaseString(stringBuffer));
	cacheAllocation += inThisChain;
      }
      cacheAllocationPercent = cacheAllocation * (100 * 100);
      cacheAllocationPercent /= PicMain::globData.cache->cacheSize * 100;
      sprintf(stringBuffer, "Cache allocation at %ld%% (%ld/%d)\n",
	      cacheAllocationPercent, cacheAllocation, 
	      PicMain::globData.cache->cacheSize);
      DEBUG_CACHE(BaseString(stringBuffer));
      // Dump PICCache contents on screen
      if(descArray != NULL) {
	free(descArray->descArray);
	free(descArray);
      }
      descArray = PicMain::globData.cache->getArrayDescr();
      DEBUG_MSG("[CACHE] Non-null elements in PICCache " << 
		descArray->nonNull << endl);
      for(unsigned int j = 0; j < descArray->elements; j += 2)
	if(descArray->descArray[j] && descArray->descArray[j + 1])
	  DEBUG_MSG("[" << descArray->descArray[j] << "," << 
		    descArray->descArray[j + 1] << "] ");
      DEBUG_MSG(endl);
    }
  }
  DEBUG_MSG("[CLNT] Thumbnails received....OK!!"<<endl);
  
  // Quit the squibbling, let's hit it, boys! (JL)
  PicMain::getGUI().setQueryResults(pics);
  
  pics.deleteAll();
}

// =============================================================

bool PICClient::PictureData(picData_msg &msg, PicPicture &exPic)
{
  DEBUG_MSG("[CLNT] In PictureData Method"<<endl);

  unsigned int imageSize=msg.picData.picData_len;
  DEBUG_MSG("[CNLT] IMAGE SIZE = " << imageSize << endl);

  // Size of the image.
  if (!imageSize) {
    cerr<<"[CLNT] No image data\n";
    return false;
  }

  DEBUG_MSG("[CNLT] READING IMAGE BLOCK " << endl);
  char *rawImage=new char[imageSize];
  memcpy(rawImage,msg.picData.picData_val,imageSize);

  DEBUG_MSG("[CNLT] READING IMAGE BLOCK - OK!!!! " << endl);

  exPic.setNull();

  // Filename of the picture.
  exPic.filename = strdup(msg.fileName);
  DEBUG_MSG("[CNLT] Filename = " << exPic.filename.chars() << endl);
  // First parameter of the picture.
  exPic.par1 = strdup(msg.paramStr1);
  DEBUG_MSG("[CNLT] Par1 = " << exPic.par1.chars() << endl);
  // Second parameter of the picture.
  exPic.par2 = strdup(msg.paramStr2);
  DEBUG_MSG("[CNLT] Par2 = " << exPic.par2.chars() << endl);

  // Format of the image.
  long formatId = msg.picFormat;
  DEBUG_MSG("[CNLT] Format ID = " << formatId << endl);
  exPic.format=PicMain::globData.getImageFormat(formatId);

  long imageWidth = msg.picWidth;
  DEBUG_MSG("[CNLT] imageWidth = " << imageWidth << endl);
  long imageHeight = msg.picHeight;
  DEBUG_MSG("[CNLT] imageHeight = " << imageHeight << endl);

  PicRawPicture *newRaw;
  if (!exPic.format) { // it's raw
    exPic.setRaw(new PicRGBPicture(imageWidth,imageHeight,
				   (unsigned char *)rawImage));
    DEBUG_MSG("[CNLT] YES RAW!" << endl);
  } else {
    PicInCharStream stream(rawImage,imageSize);
    if ((newRaw=exPic.format->readPicture(stream,imageSize)))
      exPic.setRaw(newRaw);
    DEBUG_MSG("[CNLT] NO RAW!" << endl);
  }
  delete rawImage;

  DEBUG_MSG("[CNLT] PICTURE READ OK!" << endl);
  return true;
}   

// =============================================================

void PICClient::NewPictureId(newPic_msg &msg)
{
  DEBUG_MSG("[CLNT] In NewPictureId Method..."<<endl);
  // Kuka pivitt tmn id:n kuvan tietorakenteeseen?

  // Id of the added picture.
  //=msg.picID;
} 

// =============================================================
// 
//   Clients Requests
//
// =============================================================

bool PICClient::GetMoreThumbnails(int first, int numThumbs)
{
  // PICClient::GetMoreThumbnails method "infected" by the v3 PICCache
  DEBUG_MSG("[CLNT] in GetMoreThumbnails.\n");

  //FIXME: enable that shitty little excuse for a cache
  /*long payLoad;
  if(PicMain::globData.cacheIsDisabled == false)
    payLoad = PicMain::globData.picPars->cacheSize * 2 * sizeof(long);
  else
    payLoad = 2;
    PICMessage packet(20 + payLoad);*/

  moreThumbs_msg msg;

  msg.firstThumb=first;
  msg.thumbNum=numThumbs;

  xdrH->send(&msg);
  
  /*// If the PICCache is disabled, do not attempt to use it
  if(PicMain::globData.cacheIsDisabled == false) {
    // Free the PICCache related information structure before redoing it
    if(descArray != NULL) {
      free(descArray->descArray);
      free(descArray);
    }
    // Fill the packet with PICCache information
    descArray = PicMain::globData.cache->getArrayDescr();
    // First write how many ids the server should expect
    packet.WriteInt32(descArray->nonNull);
    // Then loop through array taking only non-null values
    for(int i = 0; i < descArray->elements; i += 2) {
      if(descArray->descArray[i] || descArray->descArray[i + 1]) {
	packet.WriteInt32(descArray->descArray[i]);
	packet.WriteInt32(descArray->descArray[i + 1]);
      }
    }
  }
  else {
     packet.WriteInt32(0);
  }
  packet.Crop();
  if (!net.WritePacket(&packet))
  return false;*/

  DEBUG_MSG("[CLNT] GetMoreThumbnails message sent.\n");
  return true;
}




