// **********************************************************************
//  This file contains the image processing related routines of PicMain
//  Part of : PicCore
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <mmain.hxx>
#include <mtools.hxx>
#include <pfilestm.hxx>
#include <string.h>

#include <PICClient.h>

// **********************************************************************

PICBOOL PicMain::loadImage(BaseString filename,
                           PicPicture &resultImage,
                           PICULONG   listIndex,
                           PICBOOL    updating)
{
  PICBOOL ret = boolFalse;

  if(listIndex==0){ // it's grabbed from image editor for processing
    resultImage.setNull();
    return resultImage.loadFromFile(filename);
  }

  PicPicture *pic = getExamplePic(listIndex);
  resultImage.setNull();
  if(pic){
    if(!updating){ 
      INFO_PRINT("Opening file \"" & filename & "\"...\n");
      ret = resultImage.loadFromFile(filename);
    }else{
      DEBUG_CORE("Updating query image from file \"" & filename & "\".\n");
      ret = resultImage.loadFromFile(filename);
    }
    if(ret){
      if(!updating){
        INFO_PRINT("Image loaded from file \"" & filename & "\".\n");
        pic->setNull();
        pic->filename = resultImage.filename = filename;
        pic->format = resultImage.format;
        pic->identicalToFilename = resultImage.identicalToFilename;
        pic->origData = (PicObject &)(resultImage.origData);
      }else{
        pic->signatures.listClean();
		pic->segmentations.setNull();
        pic->origData.setNull();
        pic->identicalToFilename = boolFalse;
        resultImage.id = pic->id;
        resultImage.name = pic->name;
        resultImage.par1 = pic->par1;
        resultImage.par2 = pic->par2;
        resultImage.filename = pic->filename;
      }
      if(!pic->raw)
        pic->raw = new PicRawPicture();
      if(resultImage.raw){
        pic->raw->setWidth(resultImage.raw->getWidth());
        pic->raw->setHeight(resultImage.raw->getHeight());
      }
    }else{
      if(updating){
        INFO_PRINT("Could not update query image.\n");
      }else{
        INFO_PRINT("Could not load from file \"" & filename & "\".\n");
      }
      resultImage.setNull();
    }
  }
  FLUSH_ERRORS();
  return ret;
}

// **********************************************************************

PICBOOL PicMain::saveImage(BaseString filename,
                           PICULONG listIndex,
                           PICBOOL overwrite)
{
	PICBOOL ret = boolFalse;
	PicFile file(filename);

	if(!overwrite && file.exists()){
		return boolFalse;
	}

	INFO_PRINT("Saving image to file \"" & filename & "\"...\n");

	PicPicture *pic = getExamplePic(listIndex);

    if(!pic->origData.isNull())
    {
        ret = pic->origData.writeToFile(filename);
        if(ret)
            pic->filename = filename;
    }
    else
    {
		if(pic->raw)
		{
        	PicImageFormat *format;
			format=PicMain::globData.getImageFormat(PIC_JPEG_IMAGEFORMAT_ID);

        	if(format && (ret=format->writePicture(filename, pic->raw)))
				pic->filename=filename;
    	}
	}
	if(ret)
	{
		INFO_PRINT("Save ok.\n");
	}
	else
		INFO_PRINT("Save failed...\n");

	FLUSH_ERRORS();
	return ret;
}

// **********************************************************************

PICBOOL PicMain::setImageAttributes(PicIxCollect &params,
                                    PICULONG listIndex)
{
  PicErrorContext ecxt;
  PICBOOL ret = boolFalse;
  PICBOOL sizeChanged = boolFalse;

  PicPicture *pic = getExamplePic(listIndex);
  if(pic){
    if(params.contains(1)){
      pic->name = AS(PicString, params.at(1));
    }
    if(params.contains(2)){
      pic->par1 = AS(PicString, params.at(2));
    }
    if(params.contains(3)){
      pic->par2 = AS(PicString, params.at(3));
    }
    if(params.contains(4)){
      PicString &str = AS(PicString, params.at(4));
      PicInteger newWid(str);
      if(PicError::isDetected()){
        PicError::clearCurrent();
        str = asString(pic->raw->getWidth());
      }else{
        if(newWid<1) 
          newWid=1;
        if(newWid>1024 && !(pic->raw->getWidth()>=newWid))
          newWid=1024;
          if(newWid.asInt() != pic->raw->getWidth()){
            sizeChanged = boolTrue;
          pic->raw->setWidth(newWid.asInt());
          }
        }
    }
    
    if(params.contains(5)){
      PicString &str = AS(PicString, params.at(5));
      PicInteger newHeight(str);
      if(PicError::isDetected()){
        PicError::clearCurrent();
        str = asString(pic->raw->getHeight());
      }else{
        if(newHeight<1) 
          newHeight=1;
        if(newHeight>768 && !(pic->raw->getHeight()>=newHeight))
          newHeight=768;
        str = newHeight.getImage();
        if(newHeight.asInt() != pic->raw->getHeight()){
          sizeChanged = boolTrue;
          pic->raw->setHeight(newHeight.asInt());
        }
      }
    }
    if(sizeChanged){
      pic->signatures.listClean();
      pic->segmentations.setNull();
      pic->origData.setNull();
      pic->identicalToFilename = boolFalse;
    }
  }
  ret = !PicError::isDetected();
  FLUSH_ERRORS();
  return ret;
}

