// **********************************************************************
//  This file contains implementations of PicQueryArray and PicQueryObj 
//  Part of : PicCore
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <mquery.hxx>
#include <mmain.hxx>

MAKE_PICOBJECTCLASS(PicQueryObj, PicObject)

PicQueryObj::PicQueryObj()
  : imageId(0), matchRate(0)
{
}

PicQueryObj::PicQueryObj(PICULONG imId, PICUSHORT mRate)
  : imageId(imId), matchRate(mRate)
{
}

PicObject& PicQueryObj::operator = (const PicObject& obj)
{
  if (TYPE_GUARD(classId(), obj)){
    const PicQueryObj &other = (const PicQueryObj&)obj;
    imageId = other.imageId;
    matchRate = other.matchRate;
  }
  return *this;
}

PICBOOL PicQueryObj::operator == (const PicObject& obj) const
{
  if (TYPE_GUARD(classId(), obj)){
    const PicQueryObj &other = (const PicQueryObj&)obj;
    return (imageId == other.imageId && matchRate == other.matchRate);
  }
  return boolFalse;
}

PICBOOL PicQueryObj::operator <  (const PicObject& obj) const
{
  if (TYPE_GUARD(classId(), obj)){
    const PicQueryObj &other = (const PicQueryObj&)obj;
    return matchRate > other.matchRate;  // Note this
  }
  return boolFalse;
}

PICBOOL PicQueryObj::isNull() const
{
  return (imageId==0 && matchRate==0);
}

void PicQueryObj::setNull()
{
  imageId=0;
  matchRate=0;
}

BaseString PicQueryObj::getImage() const
{
  return asString(imageId) & ":" & asString(matchRate);
}

void PicQueryObj::setImage(const BaseString& image)
{
  undefined("PicQueryObj::setImage");
}

// **********************************************************************

MAKE_PICOBJECTCLASS(PicQueryArray, PicArray)

PicQueryArray::PicQueryArray()
  : scannedPics(0), totalPics(0), maxObjects(0)
{
  PICUSHORT maxObj = PicMain::globData.picPars->querySetSize;
  if(maxObj > 1000)
    maxObj = 1000;
  maxObjects = maxObj;
}

PicQueryArray::PicQueryArray (const PicQueryArray& other) :
  PicArray(other)
{
  maxObjects = other.maxObjects;
  scannedPics = other.scannedPics;
  totalPics = other.totalPics;
  startTime = other.startTime;
  totalTime = other.totalTime;
}

PicQueryArray::~PicQueryArray ()
{
}

PicObject& PicQueryArray::operator = (const PicObject& obj)
{ 
  if(obj.IS_A(PicQueryArray)){
    const PicQueryArray &other = AS(PicQueryArray, obj);
    PicArrayObject::operator = (obj);
    scannedPics = other.scannedPics;
    totalPics = other.totalPics;
    startTime = other.startTime;
    totalTime = other.totalTime;
  }
  return *this;
}


void PicQueryArray::setNull()
{
  PicArray::setNull();
  scannedPics = 0;
  totalPics = 0;
  startTime.setNull();
  totalTime.setNull();
}

PICBOOL PicQueryArray::add(const PicObject& obj)
{
  PICBOOL ret = PicSeqIxCollect::add(obj);
  if(ret){
    PICUSHORT nowSize = size();
    if(nowSize > maxObjects*2){
      crop();
    }
  }
  return ret;
}

void PicQueryArray::crop()
{
  PICSIZE nowSize = size();
  sort();
  while(nowSize > maxObjects){
    deleteAt(nowSize);
    nowSize--;
  }
}

void PicQueryArray::updateTotalTime()
{
  if(!startTime.isNull()){
    BaseTime interval = BaseTime(now) - startTime;
    if(totalTime.isNull()){
      totalTime = interval;
    }else{
      totalTime.addHours(interval.hour());
      totalTime.addMinutes(interval.minute());
      totalTime.addSeconds(interval.second());
    }
  }
}

