// --------------------------------------------------------------------------
//  ITERATOR IMPLEMENTATION CLASS
//  Part of : PicLib
// --------------------------------------------------------------------------
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// --------------------------------------------------------------------------

#ifndef PITERIMP_HXX
#define PITERIMP_HXX

#include <piterate.hxx>

// --------------------------------------------------------------------------
//      ITERATOR IMPLEMENTATION
/*:des ----------------------------------------------------------------------

    void    SET_ITERATE_INDEX (PicIterator&, const PICLONG index);
    PICLONG GET_ITERATE_INDEX (PicIterator&);

    void    INCREMENT_ITERATE_INDEX (PicIterator&, const PICLONG increment);
    void    DECREMENT_ITERATE_INDEX (PicIterator&, const PICLONG decrement);

These macros are used to manipulate the iterator index. They hide casting
PicIterator& into PicIteratorImp&.

*/

#define SET_ITERATE_INDEX(iterator, index) \
    ((PicIteratorImp*)(&iterator))->set (index)

#define GET_ITERATE_INDEX(iterator) \
    ((PicIteratorImp*)(&iterator))->get ()

#define INCREMENT_ITERATE_INDEX(iterator, amount) \
    ((PicIteratorImp*)(&iterator))->increment (amount)

#define DECREMENT_ITERATE_INDEX(iterator, amount) \
    ((PicIteratorImp*)(&iterator))->decrement (amount)


/*:class ====================================================================
PicIteratorImp is derived from PicIterator. It is used in implementing
iterators for individual classes. It allows the manipulation of the
iterator index. Several macros are defined to hide casting references
to PicIterator into references to PicIteratorImp.

PicIterator is always used in public interfaces of iterators. PicIteratorImp
is only intended for implementors.

    class PicIteratorImp : public PicIterator
    {
        public:

        PicIteratorImp ();

        void    set (const PICLONG index);
        PICLONG get ();

        void increment (const PICLONG amount);
        void decrement (const PICLONG amount);

    };

*/

class PicIteratorImp : public PicIterator
{

    public:
// --------------------------------------------------------------------------
//      CONSTRUCTORS

    PicIteratorImp () : PicIterator () {}

// --------------------------------------------------------------------------
//      METHODS

    void set (PICLONG index) { current = index; }
    PICLONG get () { return current; }
    void increment (PICLONG amount) { current += amount; }
    void decrement (PICLONG amount) { current -= amount; }

};

#endif
