// --------------------------------------------------------------------------
//  PicList implementation
//
//  This is a very simple list class to keep anything in it
//  which can be casted into (void *). 
//
//  A more standard STL list would have been used, but including
//  it into the PICSearch build in the compile environment of 
//  University of Helsinki (dept. of Computer Science) just didn't
//  work out. With any recent (decent) compiler tools there should
//  be no problem to use STL.
//
//  Part of : PicLib
// --------------------------------------------------------------------------
// Copyright (C) 1999 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// --------------------------------------------------------------------------

#include <PicList.hxx>
#include <iostream.h>

PicList::PicList()
{
	list_current=NULL;
	list_head=NULL;
}

PicList::~PicList()
{
	listClean();
}

void PicList::listClean()
{
	struct ListStruct *tmp;

	list_current=list_head;
	
	while(list_current)
	{
		tmp=list_current->list_next;
		delete(list_current->list_node);
		delete(list_current);
		list_current=tmp;
	}

	list_current=NULL;
	list_head=NULL;
}

void PicList::listRewind(void)
{
	list_current=NULL;
}

void *PicList::listCurrent(void)
{
	if(list_current)
		return(list_current->list_node);
	else
		return(NULL);
}

void *PicList::listNext(void)
{
	if(list_current)
	{
		list_current=list_current->list_next;
		if(list_current)
			return(list_current->list_node);
		else
			return(NULL);
	}
	else
	{
		list_current=list_head;
		if(list_current)
			return(list_current->list_node);
		else
			return(NULL);
	}
}

int PicList::listAdd(void *node)
{
	ListStruct *tmp=new ListStruct;

	tmp->list_node=node;
	tmp->list_next=list_head;
	list_head=tmp;

	return(1);
}

int PicList::listRemove(void *node)
{
	ListStruct *tmp=list_head;
	ListStruct *prev=NULL;

	while(tmp!=NULL)
	{
		if(tmp->list_node==node)
		{
			if(prev)
				prev->list_next=tmp->list_next;
			else
				list_head=tmp->list_next;
				
			delete(tmp->list_node);
			delete(tmp);

			return(1);
		}
		prev=tmp;
		tmp=tmp->list_next;
	}

	return(0);
}

