// --------------------------------------------------------------------------
//  STRING ARRAY implementation
//  Part of : PicLib
// --------------------------------------------------------------------------
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// --------------------------------------------------------------------------

extern "C" {
#include <string.h>
}
#include <piterate.hxx>
#include <pcollect.hxx>

// --------------------------------------------------------------------------
//  Edit this include:

#include <bstrarr.hxx>

// --------------------------------------------------------------------------
//      STATIC PARAMETER FUNCTIONS
// --------------------------------------------------------------------------
//  Create a new component at the address pointed to by 'to'.
//  The component should be created as null-valued.

static void createItem (BaseString* to)
{
//    new(to) BaseString();

    memset(to, 0, sizeof(BaseString));
    BaseString tmp;
    *to = tmp;
}

// --------------------------------------------------------------------------
//  Create a new component at the address pointed to by 'to'.
//  Copy the value of the argument 'from'.

static void createItemCopy (BaseString* to, const BaseString* from)
{
//    new(to) BaseString(*from);

    memset(to, 0, sizeof(BaseString));
    *to = from->copy();
}

// --------------------------------------------------------------------------
//  Return boolTrue if the given component is in null state.
//  Such components are regarded as non-existing, so locations
//  occupied by them are empty.

static PICBOOL isItemNull (const BaseString* item)
{
    return item->isNull();
}

// --------------------------------------------------------------------------
//  Set the given component in null state. After that it must not
//  allocate dynamic memory.

static void setItemNull (BaseString* item)
{
    item->setNull();
}

// --------------------------------------------------------------------------
//  Compare two components for ascending order. Return
//   0  if they are equal
//   1  if 'i1' sorts after 'i2'
//  -1  if 'i1' sorts before 'i2'
//  Make sure that null components are sorted after all others.

static int compareAsc (BaseString* i1, BaseString* i2)
{
    if (i1->isNull() || i2->isNull())
        return (i1->isNull() - i2->isNull()); // isNull is PICBOOL
    if (*i1 == *i2)
        return 0;
    return *i1 < *i2 ? -1 : 1;
}

// --------------------------------------------------------------------------
//  Compare two components for descending order. Return
//   0  if they are equal
//  -1  if 'i1' sorts after 'i2'
//   1  if 'i1' sorts before 'i2'
//  Make sure that null components are sorted after all others.

static int compareDesc (BaseString* i1, BaseString* i2)
{
    if (i1->isNull() || i2->isNull())
        return (i1->isNull() - i2->isNull()); // isNull is PICBOOL
    if (*i1 == *i2)
        return 0;
    return *i1 < *i2 ? 1 : -1;
}

// --------------------------------------------------------------------------
//      TYPE INFORMATION
// --------------------------------------------------------------------------

static PicGenArrayInfo typeInfo =
{
    sizeof(BaseString),

    (PROC_SETITEM*)      createItem,
    (PROC_SETITEM*)      NULLPTR,    // or NULLPTR to init as zeros
    (PROC_COPYITEM*)     createItemCopy,
    (PROC_SETITEM*)      setItemNull,
    (PROC_TESTITEM*)     isItemNull,
    (PROC_COMPAREITEMS*) compareAsc,
    (PROC_COMPAREITEMS*) compareDesc
};

#include <genarray.cgn>

PICBOOL BaseStringArray::add (const PicCollection& collect)
{
    PicIterator i;
    ITERATE(collect, i)
        if (!add(collect.at(i).getImage()))
            return boolFalse;
    return boolTrue;
}
