// **********************************************************************
//  This is the header file for the "Skeleton" example library
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1999 PICSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#ifndef PICSKELETON_HXX
#define PICSKELETON_HXX

#include <PicLibAlgorithm.hxx>

/*
 * Here you define a class for the internal representation 
 * of your algorithms signature. The class should be derived
 * from "PicSignature" -class.
 *
 * The only purpose of this class is to convert the signature 
 * to/from the database binary string representation to a more 
 * easily used internal one.
 *
 * Basically, you should have two types of constructors
 *
 * "PicSkeleton(PicSignature *from)" which converts an
 * incoming "binary only" signature into your internal
 * representation, to be used in calculations.
 *
 * "PicSkeleton(int something, bool other, ...)" which 
 * converts the inputted variables into a binary 
 * representation which is given onwards to the caller
 * and stored into the database.
 *
 * If a signature constructor fails, set sigFailure=1
 * and return.
 *
 *
 */

class PicSkeletonSignature : public PicSignature
{
	public:
		PicSkeletonSignature();
		PicSkeletonSignature(PicSignature *from);
		PicSkeletonSignature(int dummy, int empty);
		~PicSkeletonSignature();

		int sigFailure;

	private:
		int internalvariable1;
		int internalvariable2;
		double internaldouble1;

	friend class PicLibAlgorithm;
};

// **********************************************************************

#endif

