// **********************************************************************
//  Implementation of search algorithm "Name"
//  Part of : PicAlg
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <string.h>
#include <math.h>
#include <picerror.hxx>
#include <pstring.hxx>
#include <pinteger.hxx>

#include <PicNameAlg.hxx>
#include <mmain.hxx>

PicLibAlgorithm::PicLibAlgorithm()
{
	useName=boolTrue;
	usePar1=boolFalse;
	usePar2=boolFalse;
	
	name="NameAlg";
	info="Name algorithm\n";
}

PICBOOL PicLibAlgorithm::setParams(PicIxCollect &params)
{
  // Set your parameters from strings given by the user

  PicIterator iter;

  ITERATE(params, iter)
  {
    PicString &curr = AS(PicString, params.at(iter));
    PICINDEX  ix    = params.indexAt(iter);   // index

    switch(ix){
      case 1:
        if(curr == BaseString("1"))
          useName = boolTrue;
        else
          useName = boolFalse;
        break;
      case 2:
        if(curr == BaseString("1"))
          usePar1 = boolTrue;
        else
          usePar1 = boolFalse;
        break;
      case 3:
        if(curr == BaseString("1"))
          usePar2 = boolTrue;
        else
          usePar2 = boolFalse;
        break;
      case 4:
        imName = curr;
        break;
      case 5:
        par1 = curr;
        break;
      case 6:
        par2 = curr;
        break;
      default: // ignore
        break;
    }
  }
  
  return boolTrue;  // or boolFalse
}
  

PICUSHORT PicLibAlgorithm::compare(PicSignature &firstSig, 
                             PicSignature &secSig)
{
  // Compare the signatures and return value between 
  // 0 (worst) and 1000 (best)

  PICUSHORT retVal = 0, compsUsed = 0, i;

  PicNameAlgSignature baseSig(&firstSig);
  PicNameAlgSignature actSig(&secSig);
  if(baseSig.sigFailure || actSig.sigFailure)
      return(0);

    for(i = 0; i < 3; i++){

      BaseString name1, name2;
      PICBOOL skip = boolTrue;

      switch(i){
        case 0: 
          if(useName){
//              name1 = baseSig.imName; name2 = actSig.imName;
            name1 = imName; name2 = actSig.imName;
            skip = boolFalse;
          }
          break;
        case 1: 
          if(usePar1){
//              name1 = baseSig.par1; name2 = actSig.par1;
            name1 = par1; name2 = actSig.par1;
            skip = boolFalse;
          }
          break;
        case 2: 
          if(usePar2){
//              name1 = baseSig.par2; name2 = actSig.par2;
            name1 = par2; name2 = actSig.par2;
            skip = boolFalse;
          }
          break;
      }
      if(!skip){
        name1.trim();  name2.trim();
        if(name1 == name2){
          retVal += 1000;
        }else if(name1.equal(name2)){   // case insensitive
          retVal += 900;
        }else if(name2.positionOf(name1) != outOfRange){   // contained
          retVal += 800 - name2.size()*10 + name1.size()*10;
        }else if(name1.positionOf(name2) != outOfRange){   // contained
          retVal += 600 - name2.size()*10 + name1.size()*10;
        }
        compsUsed++;
      }
    }
  
  if(compsUsed==0)
    compsUsed = 1;

  return retVal / compsUsed;
}

PicSignature *PicLibAlgorithm::computeSignature(PicPicture &pic)
{
  // Generate a signature for the argument image

  PicNameAlgSignature *tmp;
  
  tmp=new PicNameAlgSignature(pic.name, pic.par1, pic.par2);
  if(tmp->sigFailure)
  	return(0);

  return tmp;
}

/*
BaseString ALG_CLASS::getImage() const
{
  return name & " [" & asString(id) & "]";
}
*/

// **********************************************************************

// Default dummy constructor referring

PicNameAlgSignature::PicNameAlgSignature()
{
	sigFailure=0;
}

PicNameAlgSignature::~PicNameAlgSignature()
{ 
}

PicNameAlgSignature::PicNameAlgSignature(BaseString newName, BaseString newPar1, BaseString newPar2) :
  imName(newName), par1(newPar1), par2(newPar2)
{
  PICULONG flen;
  PICULONG allocSize = imName.size()+par1.size()+par2.size()+3*sizeof(PICULONG);
  char *temp, *data = new char[allocSize];

  sigFailure=0;

  if(!data)
  {
  	sigFailure=1;
	return;
  }
  
    temp = data;

    flen = imName.size();
    memcpy(temp, &flen, sizeof(flen));
    temp += sizeof(flen);
    memcpy(temp, imName.chars(), flen);
    temp += flen;

    flen = par1.size();
    memcpy(temp, &flen, sizeof(flen));
    temp += sizeof(flen);
    memcpy(temp, par1.chars(), flen);
    temp += flen;

    flen = par2.size();
    memcpy(temp, &flen, sizeof(flen));
    temp += sizeof(flen);
    memcpy(temp, par2.chars(), flen);

  this->signature=data;
  this->signature_size=allocSize;
	
}

PicNameAlgSignature::PicNameAlgSignature(PicSignature *from)
{
	sigFailure=0;
	this->signature=NULL;
	this->signature_size=0;

	char *data=from->signature;
	PICULONG size=from->signature_size;

  if(!data){
    sigFailure=1;
	return;
  }
  const char *temp = data;
  PICULONG flen, read = 0;

  // Read name
  if(read+sizeof(flen) > size)
  {
  	sigFailure=1;
	return;
  }
  memcpy(&flen, temp, sizeof(flen));
  temp += sizeof(flen); 
  read += sizeof(flen);

  if(read+flen > size)
  {
  	sigFailure=1;
    return;
  }
  imName.setChars(temp, flen);
  temp += flen; 
  read += flen;

  // Read par1
  if(read+sizeof(flen) > size)
  {
    sigFailure=1;
	return;
  }
  memcpy(&flen, temp, sizeof(flen));
  temp += sizeof(flen); 
  read += sizeof(flen);

  if(read+flen > size)
  {
  	sigFailure=1;
    return;
  }
  par1.setChars(temp, flen);
  temp += flen; 
  read += flen;

  // Read par2
  if(read+sizeof(flen) > size)
  {
    sigFailure=1;
    return;
  }
  memcpy(&flen, temp, sizeof(flen));
  temp += sizeof(flen); 
  read += sizeof(flen);

  if(read+flen > size)
  {
  	sigFailure=1;
	return;
  }
  par2.setChars(temp, flen);
}

/*
BaseString SIG_CLASS::getImage() const
{
  return "Name = \"" & imName & 
         "\"\nDesc1 = \"" & par1 & 
         "\"\nDesc2 = \"" & par2 & "\"";
}
*/

// **********************************************************************
