// **********************************************************************
//
//      PICNet.cpp
//
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//
// General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************    

#include <PICNet.h>
#include <signal.h>

//=======================================================================
// Tries to connect to the host.  
//=======================================================================

int PICNet::serverSocketID(void) { 
  return(serverSocket);
}

bool PICNet::ClientSocketInit(char *hostName, int portNumber)
{
  struct sockaddr_in  serverSockAdr;
  struct hostent *hostptr;

  if (portNumber < 1 || portNumber > MAX_PORT){
    cout << "[PICNet] Invalid port number.\n" << endl;
    return false;
  }
  serverSockAdr.sin_family = AF_INET;
  serverSockAdr.sin_port = htons (portNumber);

  // hostName will be intepreted as a name 
  hostptr = gethostbyname (hostName); 
  if (hostptr != NULL)
    bcopy (hostptr->h_addr, (char *) &serverSockAdr.sin_addr, hostptr->h_length);

  // hostName will be intepreted as an addres
  else {
    serverSockAdr.sin_addr.s_addr = inet_addr (hostName);
    if (serverSockAdr.sin_addr.s_addr == -1) {
      cout << "[PICNet] Unvalid hostaddres!  Hostname: " << hostName << endl;
      return false;
    }
  }

  serverSocket = socket (AF_INET, SOCK_STREAM, 0);
  if (serverSocket == -1) {
    cout << "[PICNet] Couldn't open socket!" << endl;
    return false;
  }
  
  if (connect (serverSocket, (struct sockaddr *)&serverSockAdr, sizeof (serverSockAdr)) == -1) {
    cout << "[PICNet] Couldn't connect to the Host!" << endl;
    return false;
  }
  return true;
}

//=======================================================================
// Sets up the host socket
//=======================================================================

bool PICNet::HostSocketInit(int portNumber) {
  int adrlen;

  if (portNumber < 1 || portNumber > MAX_PORT){
    cout << "[PICNet] Invalid port number.\n" << endl;
    return false;
  }
  
  hostSocket = socket (AF_INET, SOCK_STREAM, 0);  
  if (hostSocket == -1) {
    cout << "[PICNet] Couldn't open host socket!" << endl;
    return false;
  }
  
  if (fcntl (hostSocket, F_SETFL, O_NONBLOCK || fcntl (hostSocket, F_GETFL)) == -1 ) {
    cout << "Couldn't change to the nonblock-mode!" << endl;
    return false;
  }	
  
  mySockAdr.sin_family = AF_INET;
  mySockAdr.sin_addr.s_addr = 0;
  mySockAdr.sin_port = htons(portNumber);
  
  adrlen = sizeof (mySockAdr);
  
  if (bind (hostSocket, (struct sockaddr *) &mySockAdr, adrlen) == -1){
    cout << "[PICNet] Couldn't bind socket!" << endl;
    return false;
  }                                                 
  
  if (listen (hostSocket, 5) == -1){
    cout << "[PICNet] Couldn't issue Listen-Cmd!" << endl;
    return false;
  }
  return true;
}

//=======================================================================
// The destructor
//=======================================================================

PICNet::~PICNet() {
  close (serverSocket);
  close (hostSocket);
}

bool PICNet::CheckSocket(int sec) {
  fd_set rfds;
  struct timeval tv;
  int retval;
  
  // Watch serverSocket to see when it has input. 
  
  FD_ZERO(&rfds);
  FD_SET(serverSocket, &rfds);
  
  // Wait up to sec seconds.
  
  tv.tv_sec = sec;
  tv.tv_usec = 1;
  
  retval = select(FD_SETSIZE, &rfds, NULL, NULL, &tv);
  /* Don't rely on the value of tv now! */
  
  if (retval == -1 ) {
    cout << "[PICNet] Error in select." << endl; 
    return false;
  }
  
  if ( FD_ISSET(serverSocket, &rfds))
    return true;
  else 
    return false;
}         


//=======================================================================
// Waits for requests from clients. Creates a new socket for every client
//=======================================================================

bool PICNet::AcceptClientRequest() {  
  int adrlen;
  
  adrlen=sizeof(mySockAdr);
again:
  errno = 0;
  cout << "[PICNet] Waiting in accept.." << endl;
  serverSocket=accept(hostSocket,(struct sockaddr *)&mySockAdr,&adrlen);
  if (errno == EINTR)  // accept interrupted, have to wake up again 
    goto again;
  if (serverSocket == -1) {
    fprintf (stderr, "Error on Accept\n\tError: %d\n", errno);
    fprintf (stderr, "\t%s\n", strerror (errno));
    
    cout << "[PICNet] Error on Accept !" << endl;
    return false;
  }
  return true;
}

//=======================================================================
// Closes the hostSocket
//=======================================================================

void PICNet::CloseHostSocket() {
  close(hostSocket);
}

//=======================================================================
// Closes the clientSocket
//=======================================================================

void PICNet::CloseServerSocket() {
  close(serverSocket);
}
