#ifndef CFGREAD_H
#define CFGREAD_H

#include<string>
#include<fstream.h>
#include<stdlib.h>
#include<map.h>
#include<ctype.h>
#include<message.h>

#define MAXCONFIGLINE 1024

int stoi(const string&); //this should be someplace else.
string itos(int);  //ditto.

typedef map<string, string, less<string> > stringMap; //maps strings to strings
typedef map<string, stringMap, less<string> > stringMapMap; //go figure.

istream& skipSpace(istream& is);  //removes space from an istream.
string removeBlanks(string s);  //returns a subs of s sans surronding spaces.

class cfgIteratorBase {
 protected:
  stringMapMap::iterator rep;
 public:
  cfgIteratorBase(stringMapMap::iterator i);
  cfgIteratorBase& operator++(void);
  cfgIteratorBase  operator++(int);
  bool operator==(const cfgIteratorBase& rhs) const;
  bool operator!=(const cfgIteratorBase& rhs) const;
  //dereferencing in subclasses.
};

class cfgIterator : public cfgIteratorBase {
 public:
  cfgIterator(stringMapMap::iterator i);
  const string& operator*(void) const;
  const string& name(void) const;
};

class cfgAlgIterator : public cfgIteratorBase {
 public:
  cfgAlgIterator(stringMapMap::iterator i);
  algorithmID_t operator*(void) const;
  algorithmID_t ID(void) const;
};

class configSection; //forward decl. needed by configFile.

class configFile {
 protected:
  string fileName;
  stringMapMap theMap;  //maps stringMaps to strings
  void writeToCfg(const string what);
public:
  configFile(string fName);
  ~configFile(void);

  void save(void);
  void load(void);
  void clear(void);
  void reload(void);

  //delegate all these operations to the map.
  cfgIterator begin(void);
  cfgIterator end(void);
  cfgIterator find(const string& k);
  bool exists(const string& k);
  stringMapMap::size_type size(void);
  //operator [] defined in sublasses.

  int remove(const string& sec);
  int remove(const string& sec,const string& var);
  void set(const string& sec,const string& var,const string& val);
  string get(const string& sec,const string& var) const;
};

class configSection {
 protected:
  const string name;
  class configFile& cfg;
 public:
  configSection(const string& n,configFile& c);
  //delegate these operations.
  int remove(void);
  int remove(const string& var);
  void set(const string &var,const string &val);
  string get(const string &var) const;
};

//the template parameter specifies the contents of the cfg sections. It should
//be of type configSection.
template<class T> class typedConfigFile : public configFile {
 public:
  typedConfigFile(string fName) : configFile(fName) { }
  T operator[](const string& k) { return T(k,*this); }
};

class configVariable {
 protected:
  const string name;
  configSection& section;
 public:
  configVariable(const string& n,configSection& s);
  int remove(void);
  //set functions cannot be declared here.
};

// this should get configVariable template params.
template<class T>class configArray {
 protected:
  const string prefix;
  configSection& section;
 public:
  //FIXME: compiler trouble.
  //configArray(const string& p,configSection& s);
  configArray(const string& p,configSection& s):
    prefix(p), section(s) { }
  T operator[](const string& n) { return T(prefix+n,section); }
  //FIXME: WARNING: this is a hack. I just wanted to compile asap and added
  //this as a quick workaround.
  T operator[](const int i) { return T(prefix+itos(i),section); }
};

template<class T>class cfgNumVar : public configVariable {
 public:
  //FIXME: compiler problems here, trying to fix..
  //cfgNumVar(const string& n,configSection& s);
  cfgNumVar(const string& n,configSection& s) : 
    configVariable(n,s) { }
  void operator=(const T n) const { section.set(name,itos((int)n)); }
  operator T() const { return (T)stoi(section.get(name)); }
};

class cfgStrVar : public configVariable {
 public:
  cfgStrVar(const string& n,configSection& s);
  void operator=(const string& s);
  operator string() const;
};

/* these are _very_ picsearch specific, but show the generalway in which
 * the configFile system should be used. */

class databaseSection : public configSection {
 public:
  databaseSection(const string& n,configFile& c);
  cfgNumVar<databaseID_t> ID;
  cfgStrVar dir;
  configArray<cfgNumVar<bool> >algs;
};

class userSection : public configSection {
 public:
  userSection(const string& n,configFile& c);
  cfgNumVar<ServerAuth> authLevel;
  cfgNumVar<bool> changePass;
  cfgStrVar password;
  configArray<cfgNumVar<DatabaseAuth> >dbAuth;
};

//FIXME: this class bloated out of all porpotion. Fix it if you can.
class algorithmSection : public configSection {
 public:
  algorithmSection(const string& n,configFile& c);
  cfgNumVar<int> algClass;
  cfgNumVar<bool> paramWin;
  cfgNumVar<bool> active;
  cfgNumVar<algorithmID_t> replacedWithID;
};

class parameterSection : public configSection {
 public:
  parameterSection(const string& n,configFile& c);
  cfgStrVar http;
  cfgStrVar info;
  cfgNumVar<unsigned int> maxclients;
  cfgNumVar<unsigned int> timeout;
};

//just shorthand, really.
typedef typedConfigFile<databaseSection> DBConfigFile;
typedef typedConfigFile<userSection> userConfigFile;
typedef typedConfigFile<parameterSection> paramConfigFile;

//this is a separate class, indexing by ID was necessary.
//typedef typedConfigFile<algorithmSection> algoConfigFile;

class algoConfigFile : public configFile {
 public:
  algoConfigFile(string fName) : configFile(fName) { }
  algorithmSection operator[](const algorithmID_t k);

  //override these for some moderate type-safety.
  cfgAlgIterator begin(void);
  cfgAlgIterator end(void);
  cfgAlgIterator find(algorithmID_t k);
  bool exists(algorithmID_t k);
};

#endif /* !CFGREAD_H */








