// **********************************************************************
//
//      Picture.cpp
//
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//
// General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <PICDebug.h>
#include <PICNet.h>
#include <PICServer.h>
#include <pstrstm.hxx>
#include <mmain.hxx> // for the V1 stuff...

//FIXME: the shit below leaks memory.
void PICServer::InsertPicture(insertPic_msg &msg) {
  DEBUG_MSG("[SRV] in InsertPicture\n");

  char *name=strdup(msg.fileName);
  long data_size=msg.picData.picData_len;
  char *data=new char[data_size];
  memcpy(data,msg.picData.picData_val,data_size);
  char *par1=strdup(msg.paramStr1);
  char *par2=strdup(msg.paramStr2);
  databaseID_t db_id=msg.picDB;
  pictureID_t img_id=msg.picID;

  DEBUG_MSG("[SRV] received image "<<img_id<<" '"<<name<<"' (size = "<< \
	    data_size<<" bytes) to be inserted to db "<<db_id<<" (par1='"<< \
	    par1<<"', par2='"<<par2<<"')\n");

  if (!CheckAuth(DB_WriteAccess,db_id)) {
    ERROR_MSG("[SRV] '"<<user_name<<"' tried to insert picture to db "<< \
	      db_id<<" with no write access\n");
    SendFail();
    return;
  }

  if (!data_size || !data) {
    DEBUG_MSG("[SRV] image being inserted has no image data\n");
    SendFail();
    return;
  } 

  imageDB *imgDB;
  if((imgDB=DB.getImgDB(db_id))==NULL) {
    ERROR_MSG("[SRV] attempt to insert picture to "<<
	      "non-existent db "<<db_id<<endl);
    SendFail();
    return;
  }

  image saveImg;
  saveImg.size=data_size;
  saveImg.img=data;

  if(!imgDB->saveImage(name,saveImg)) {
    ERROR_MSG("[SRV] failed to save image"<<endl);
    SendFail();
    return;
  }

  DEBUG_MSG("[SRV] save ok\n");
      
  DEBUG_MSG("[SRV] initializing PicPicture object\n");

  PicPicture pic;
  pic.loadFromFile(imgDB->imagePathName(name).c_str());
      
  pic.id=img_id;
  pic.name=name;
  pic.par1=par1;
  pic.par2=par2;
      
  DEBUG_MSG("[SRV] creating thumbnail image\n");

  BaseString strBuffer;
  PicRawPicture *scaled=
    PicTools::scaleImage(*(pic.raw), 
			 PicMain::globData.params.stampWidth, 
			 PicMain::globData.params.stampHeight);

  PicStringStream stream(strBuffer);
  PicImageFormat *thumbnailFormat =
    PicMain::globData.params.formatForScaledImages;

  if(!thumbnailFormat->writePicture(stream, scaled)) {
    DEBUG_MSG("[SRV] failed to scale picture"<<endl);
    SendFail();
    return;
  }
      
  DEBUG_MSG("[SRV] calculating signatures\n");
      
  if(!PicMain::calcAllSignatures(pic)) {
    DEBUG_MSG("[SRV] PicMain::calcAllSignatures failure!\n");
    SendFail();
    return;
  }

  if (!img_id) { //ID 0 means 'insert, not replace'.
    img_id=DB.newPicID(db_id); //get a new ID number.
  }

  unsigned int thumbSize = strBuffer.size();
  const char *thumbNail = strBuffer.chars();

  DEBUG_MSG("[SRV] inserting picture\n");
  
  if (!imgDB->storePic(img_id,name,par1,par2,thumbSize,thumbNail)) {
    ERROR_MSG("[SRV] InsertPicture(): db failed to comply\n");
    SendFail();
    return;
  }
  
  // iterate and insert all signatures:
  PicSignature *sigtmp;
  pic.signatures.listRewind();
  while((sigtmp=(PicSignature *)pic.signatures.listNext())) {
    // Algorithm id is sig->creator_id
    algorithmID_t algID=(algorithmID_t)sigtmp->creator_id;
    unsigned int sigSize=sigtmp->signature_size;
    char *sig=sigtmp->signature;

    DEBUG_MSG("[SRV] inserting algorithm "<< algID << " signature\n");

    if (!imgDB->storeSig(algID,img_id,sigSize,sig)) {
      ERROR_MSG("[SRV] InsertPicture: storeSig: db failed to comply\n");
      SendFail();
      return;
    }
  }

  newPic_msg reply;
  reply.picID=img_id;
  xdrH->send(&reply);

  LOG_MSG("[SRV] new picture id message sent\n");
  
  delete[] data;
  free(name);
  free(par1);
  free(par2);
}

int PICServer::GenerateFileName(char **name_ptr) const {
  char char_buf[]="0123456789abcdefghijklmnopqrstuvwxyz_";
  char *current_name=new char[strlen(*name_ptr)+1];
  strcpy(current_name,*name_ptr);
  long chg_pos=strrchr(current_name,'/')-current_name+1,rpl_pos=-1;
  bool name_ok=true;
  do {  
    DEBUG_MSG("[SRV] executing stat '"<<current_name<<"'\n");
    struct stat stat_buf;
    if (!stat(current_name,&stat_buf)) {
      DEBUG_MSG("[SRV] file already exists\n");
      name_ok=false;
      // tried everything for current character already?
      if (current_name[chg_pos]=='_') { 
	current_name[chg_pos]=(*name_ptr)[chg_pos];
	chg_pos++;
	rpl_pos=0;
	// tried every character already?
	if (current_name[chg_pos]==0||current_name[chg_pos]=='.') {
	  delete current_name;
	  return 0; // false -> failure
	}
      } else
	rpl_pos++;
      current_name[chg_pos]=char_buf[rpl_pos];
    } else {
      DEBUG_MSG("[SRV] file not found - selecting '"<<current_name<<"'\n");
      name_ok=true;
    }
  } while (!name_ok);
  if (name_ok) {
    strcpy(*name_ptr,current_name);
    delete current_name;
    return 1;
  }
  delete current_name;
  return 0; // failure
}

//leaks mem, a lot.
void PICServer::SendPicture(getPic_msg &msg) {
  DEBUG_MSG("[SRV] in SendPicture\n");

  databaseID_t db_id=msg.picDB;
  pictureID_t img_id=msg.picID;

  if(!CheckAuth(DB_ReadAccess,db_id)) {
    ERROR_MSG("[SRV] '" << user_name << "' tried to get image " << img_id <<
	      "from db "<<db_id<<" with no read access\n");
    SendFail();
    return;
  }

  DEBUG_MSG("[SRV] fetching image "<<img_id<<" from database "<<db_id<<endl);

  imageDB *imgDB;
  if((imgDB=DB.getImgDB(db_id))==NULL) {
    ERROR_MSG("[SRV] attempt to fetch image from non-existent db "<< 
	      db_id<<endl);
    SendFail();
    return;
  }

  picture *storePic;
  if((storePic=imgDB->fetchPic(img_id))==NULL) {
    ERROR_MSG("[SRV] attempt to fetch non-existent image "<< img_id << endl);
    SendFail();
    return;
  }

  picData_msg reply;

  reply.fileName=storePic->fileName;
  reply.paramStr1=storePic->paramStr1;
  reply.paramStr2=storePic->paramStr2;
  reply.picID=img_id;

  image img=imgDB->loadImage(reply.fileName);
  if(!img.img) {
    ERROR_MSG("[SRV] failed to load image data" << endl);
    SendFail();
    return;
  }

  reply.picData.picData_len=img.size;
  reply.picData.picData_val=img.img;

  PicPicture pic;
  pic.loadFromFile(imgDB->imagePathName(reply.fileName).c_str());

  reply.picFormat=pic.format->id;
  reply.picWidth=pic.raw->getWidth();
  reply.picHeight=pic.raw->getHeight();

  xdrH->send(&reply);
}

//this one works so-and-so.
void PICServer::DeletePicture(deletePic_msg &msg) {
  DEBUG_MSG("[SRV] in DeletePicture\n");

  databaseID_t db_id=msg.picDB;
  pictureID_t img_id=msg.picID;

  if (!CheckAuth(DB_WriteAccess,db_id)) {
    ERROR_MSG("[SRV] '"<<user_name<<"' tried to delete image "<<img_id<< \
	      " from db "<<db_id<<" with to write access\n");
    SendFail();
  }

  DEBUG_MSG("[SRV] deleting image "<<img_id<<" from database "<<db_id<<endl);

  imageDB *imgDB;
  if((imgDB=DB.getImgDB(db_id))==NULL) {
    ERROR_MSG("[SRV] attempt to delete image from non-existent db " << 
	      db_id << endl);
    SendFail();
    return;
  }
  
  if(!imgDB->removePic(img_id)) {
    ERROR_MSG("[SRV] failed to remove picture from DB.\n");
    SendFail();
    return;
  }

  //FIXMEs:
  //remove the actual picture file, please.
  //remove all signatures referring to the file.

  SendComplete();
}











