// **********************************************************************
//
//      Translate.cpp
//
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//
// General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#include <PICDebug.h>
#include <PICNet.h>
#include <PICServer.h>
#include <mess.h>
#include <message.h>

int PICServer::xdrTranslate(void) {
  cout << "in xdrTranslate. foo!" << endl;
  switch(xdrH->currentMessage()) {

  case msg_query:
    query_msg queryMsg;
    LOG_MSG("[XDR] received query\n"); 
    xdrH->receive(&queryMsg);
    Query(queryMsg);
    xdrH->free(&queryMsg);
    break;

  case msg_stopQuery:
    stopQuery_msg stopQueryMsg;
    LOG_MSG("[XDR] received stopQuery\n");
    xdrH->receive(&stopQueryMsg);
    StopQuery();
    xdrH->free(&stopQueryMsg);
    break;

  case msg_moreThumbs:
    moreThumbs_msg moreThumbsMsg;
    LOG_MSG("[XDR] received moreThumbs\n"); 
    xdrH->receive(&moreThumbsMsg);
    SendMoreNails(moreThumbsMsg);
    xdrH->free(&moreThumbsMsg);
    break;

  case msg_insertPic:
    insertPic_msg insertPicMsg;
    LOG_MSG("[XDR] received insertPic\n"); 
    xdrH->receive(&insertPicMsg);
    InsertPicture(insertPicMsg);
    xdrH->free(&insertPicMsg);
    break;

  case msg_getPic:
    getPic_msg getPicMsg;
    LOG_MSG("[XDR] received getPic\n"); 
    xdrH->receive(&getPicMsg);
    SendPicture(getPicMsg);
    xdrH->free(&getPicMsg);
    break;

  case msg_deletePic:
    deletePic_msg deletePicMsg;
    LOG_MSG("[XDR] received deletePic\n"); 
    xdrH->receive(&deletePicMsg);
    DeletePicture(deletePicMsg);
    xdrH->free(&deletePicMsg);
    break;

  case msg_disconnect:
    disconnect_msg disconnectMsg;
    LOG_MSG("[XDR] received disconnect\n"); 
    xdrH->receive(&disconnectMsg);
    xdrH->free(&disconnectMsg);
    return 0; //FIXME: this is an ugly hack to quit.. do better.
    break;

  case msg_modifyAlgs:
    modifyAlgs_msg modifyAlgsMsg;
    LOG_MSG("[XDR] received modifyAlgs\n"); 
    xdrH->receive(&modifyAlgsMsg);
    AdjustAlgorithms(modifyAlgsMsg);
    xdrH->free(&modifyAlgsMsg);
    break;

  case msg_getUsers:
    getUsers_msg getUsersMsg;
    LOG_MSG("[XDR] received getUsers\n"); 
    xdrH->receive(&getUsersMsg);
    SendUsers();
    xdrH->free(&getUsersMsg);
    break;

  case msg_changeRights:
    changeRights_msg changeRightsMsg;
    LOG_MSG("[XDR] received changeRights\n"); 
    xdrH->receive(&changeRightsMsg);
    ChangeUserRights(changeRightsMsg);
    xdrH->free(&changeRightsMsg);
    break;

  case msg_addUser:
    addUser_msg addUserMsg;
    LOG_MSG("[XDR] received addUser\n"); 
    xdrH->receive(&addUserMsg);
    AddUser(addUserMsg);
    xdrH->free(&addUserMsg);
    break;

  case msg_deleteUser:
    deleteUser_msg deleteUserMsg;
    LOG_MSG("[XDR] received deleteUser\n"); 
    xdrH->receive(&deleteUserMsg);
    RemoveUser(deleteUserMsg);
    xdrH->free(&deleteUserMsg);
    break;

  case msg_changePW:
    changePW_msg changePWMsg;
    LOG_MSG("[XDR] received changePW\n"); 
    xdrH->receive(&changePWMsg);
    ChangePassword(changePWMsg);
    xdrH->free(&changePWMsg);
    break;

  case msg_createDB:
    createDB_msg createDBMsg;
    LOG_MSG("[XDR] received createDB\n");
    xdrH->receive(&createDBMsg);
    CreateDB(createDBMsg);
    xdrH->free(&createDBMsg);
    break;

  case msg_deleteDB:
    deleteDB_msg deleteDBMsg;
    LOG_MSG("[XDR] received deleteDB\n");
    xdrH->receive(&deleteDBMsg);
    DeleteDB(deleteDBMsg);
    xdrH->free(&deleteDBMsg);
    break;

  case msg_getServOpt:
    getServOpt_msg getServOptMsg;
    LOG_MSG("[XDR] received getServOpt\n"); 
    xdrH->receive(&getServOptMsg);
    SendOptions();
    xdrH->free(&getServOptMsg);
    break;

  case msg_setServOpt:
    setServOpt_msg setServOptMsg;
    LOG_MSG("[XDR] received setServOpt\n"); 
    xdrH->receive(&setServOptMsg);
    SetOptions(setServOptMsg);
    xdrH->free(&setServOptMsg);
    break;

  case msg_addAlg:
    addAlg_msg addAlgMsg;
    LOG_MSG("[XDR] received addAlg\n"); 
    xdrH->receive(&addAlgMsg);
    addAlgorithm(addAlgMsg);
    xdrH->free(&addAlgMsg);
    break;

  case msg_modifyAlg:
    modifyAlg_msg modifyAlgMsg;
    LOG_MSG("[XDR] received modifyAlg\n"); 
    xdrH->receive(&modifyAlgMsg);
    modifyAlgorithms(modifyAlgMsg);
    xdrH->free(&modifyAlgMsg);
    break;

  case msg_removeAlg:
    removeAlg_msg removeAlgMsg;
    LOG_MSG("[XDR] received removeAlg\n"); 
    xdrH->receive(&removeAlgMsg);
    removeAlgorithm(removeAlgMsg);
    xdrH->free(&removeAlgMsg);
    break;

  default:
    LOG_MSG("[XDR] unexpected message.\n");
    xdrH->proceed();
    break;

    /*case msg_: //TEMPLATE
    _msg Msg;
    LOG_MSG("[XDR] received \n"); 
    xdrH->receive(&Msg);
    (Msg);
    xdrH->free(&Msg);
    break;*/
  }   
  return 1;
}
