// **********************************************************************
//  PicMain class definition
//  Part of : PicCore
// **********************************************************************
// Copyright (C) 1997 PicSearch Team
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
//
// General Public License for more details. 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
//
// For any comments please contact: klemstro@cs.helsinki.fi
//
// **********************************************************************

#ifndef MMAIN_HXX
#define MMAIN_HXX

#include <palloc.hxx>

#include <PicSketchAlg.hxx>
#include <PicSegmentAlgorithm.hxx>
#include <PicImageFormat.hxx>

#include <mshared.hxx>
#include <merrors.hxx>
#include <mthreads.hxx>
#include <mtools.hxx>

/**
  * PicMain is a placeholder class full of static member functions, 
  * most of which are called by PicGUI. One PicMain object is 
  * constructed in the main-function to initialize the system.
  **/

class PicMain
{
 public:

  PICCLASS()
  PICERRORCODES {  ok,
    initFailed,
    unKnownImageFormat,
    unKnownSearchType,
    unKnownSearchAlg,
    unKnownSegmAlg,
    unKnownIPAlg,
    unKnownExampleImage,
    threadSuspendFailure,
    threadAbortFailure,
    invalidParams,
    imageInsertFailed,
    imageReadFailed,
    deleteImageFailed,
    exImageCreateFailed,
    exImageCloseFailed,
    exImageClearFailed
  };

  /**
    * Constructor initializes the system by creating 
    * search types 
    * and available 
    * PicImageFormats, 
    * PicSearchAlgorithms and 
    * PicSegmentAlgorithms 
    **/
  PicMain();
  /**
    * The destructor.
    **/
  ~PicMain();

//:var
  static PicMainSharedData globData;
/*
PicMain::globData contains the publicly available shared data.

see: PicMainSharedData
*/

//:fun
  static PICBOOL calcAllSegments(PicPicture &pic);
/*
Computes segmentations with all known (isAutomatic) segmentation
algorithms.
*/
//:fun
  static PICBOOL calcAllSignatures(PicPicture &pic);
/*
Computes signatures with all known search algorithms.
*/

//:fun
  static PICBOOL checkSegmAlgInDb(PicSegmentAlgorithm &alg);
  static PICBOOL checkSearchAlgInDb(PicSearchAlgorithm &alg);
/*
Check whether a given algorithm is found in the database.
*/

  static void printErrors(void);
/*
Used to print to info screen.
*/

	static void debugMesgAlg(BaseString mesg) {};
};

#endif
